@@ -50,7 +50,10 @@ def startQuickAsy():
     pass
   try:
     quickAsyFailed = False
-    AsyTempDir=mkdtemp(prefix="asy_")+os.sep
+    if os.name == "nt":
+      AsyTempDir=mkdtemp(prefix="asy_", dir="./")
+    else:
+      AsyTempDir=mkdtemp(prefix="asy_")+os.sep
     quickAsy = Popen([xasyOptions.options['asyPath']]+
                      split("-noV -multiline -interactive -o"+AsyTempDir),
                      stdin=PIPE,stdout=PIPE,stderr=STDOUT)
@@ -353,7 +356,7 @@ class asyPath(asyObj):
     syncQuickAsyOutput()
     quickAsy.stdin.write("path p="+self.getCode()+';\n')
     quickAsy.stdin.write("write(length(p));\n")
-    quickAsy.stdin.write("write(p);write(\"\");\n")
+    quickAsy.stdin.write("write(unstraighten(p));write(\"\");\n")
     quickAsy.stdin.flush()
     lengthStr = quickAsy.stdout.readline()
     pathSegments = eval(lengthStr.split()[-1])
