@@ -15,6 +15,7 @@
 #include "types.h"
 #include "fileio.h"
 #include "callable.h"
+#include "mathop.h"
 
 namespace run {
 
@@ -26,6 +27,30 @@ using camp::tab;
 vm::array *copyArray(vm::array *a);
 vm::array *copyArray2(vm::array *a);
   
+template<class T, class U, template <class S> class op>
+void arrayOp(vm::stack *s)
+{
+  U b=pop<U>(s);
+  array *a=pop<array*>(s);
+  size_t size=checkArray(a);
+  array *c=new array(size);
+  for(size_t i=0; i < size; i++)
+    (*c)[i]=op<T>()(read<T>(a,i),b,i);
+  s->push(c);
+}
+
+template<class T, class U, template <class S> class op>
+void opArray(vm::stack *s)
+{
+  array *a=pop<array*>(s);
+  T b=pop<T>(s);
+  size_t size=checkArray(a);
+  array *c=new array(size);
+  for(size_t i=0; i < size; i++)
+    (*c)[i]=op<U>()(b,read<U>(a,i),i);
+  s->push(c);
+}
+
 template<class T, template <class S> class op>
 void arrayArrayOp(vm::stack *s)
 {
@@ -34,31 +59,7 @@ void arrayArrayOp(vm::stack *s)
   size_t size=checkArrays(a,b);
   array *c=new array(size);
   for(size_t i=0; i < size; i++)
-      (*c)[i]=op<T>()(read<T>(a,i),read<T>(b,i),i);
-  s->push(c);
-}
-
-template<class T, template <class S> class op>
-void arrayOp(vm::stack *s)
-{
-  T b=pop<T>(s);
-  array *a=pop<array*>(s);
-  size_t size=checkArray(a);
-  array *c=new array(size);
-  for(size_t i=0; i < size; i++)
-      (*c)[i]=op<T>()(read<T>(a,i),b,i);
-  s->push(c);
-}
-
-template<class T, template <class S> class op>
-void opArray(vm::stack *s)
-{
-  array *a=pop<array*>(s);
-  T b=pop<T>(s);
-  size_t size=checkArray(a);
-  array *c=new array(size);
-  for(size_t i=0; i < size; i++)
-      (*c)[i]=op<T>()(b,read<T>(a,i),i);
+    (*c)[i]=op<T>()(read<T>(a,i),read<T>(b,i),i);
   s->push(c);
 }
 
@@ -121,11 +122,11 @@ void binopArray2(vm::stack *s)
     size_t aisize=checkArray(ai);
     if(aisize) {
       if(empty) {
-	m=read<T>(ai,0);
-	empty=false;
+        m=read<T>(ai,0);
+        empty=false;
       }
       for(size_t j=0; j < aisize; j++)
-	m=op<T>()(m,read<T>(ai,j));
+        m=op<T>()(m,read<T>(ai,j));
     }
   }
   if(empty) vm::error(arrayempty);
@@ -146,13 +147,13 @@ void binopArray3(vm::stack *s)
       array *aij=read<array*>(ai,j);
       size_t aijsize=checkArray(aij);
       if(aijsize) {
-	if(empty) {
-	  m=read<T>(aij,0);
-	  empty=false;
-	}
-	for(size_t k=0; k < aijsize; k++) {
-	  m=op<T>()(m,read<T>(aij,k));
-	}
+        if(empty) {
+          m=read<T>(aij,0);
+          empty=false;
+        }
+        for(size_t k=0; k < aijsize; k++) {
+          m=op<T>()(m,read<T>(aij,k));
+        }
       }
     }
   }
@@ -160,6 +161,40 @@ void binopArray3(vm::stack *s)
   s->push(m);
 }
 
+template<class T>
+bool Array2Equals(vm::stack *s)
+{
+  array *b=pop<array*>(s);
+  array *a=pop<array*>(s);
+  size_t n=checkArray(a);
+  if(n != checkArray(b)) return false;
+  if(n == 0) return true;
+  size_t n0=checkArray(read<array*>(a,0));
+  if(n0 != checkArray(read<array*>(b,0))) return false;
+    
+  for(size_t i=0; i < n; ++i) {
+    array *ai=read<array*>(a,i);
+    array *bi=read<array*>(b,i);
+    for(size_t j=0; j < n0; ++j) {
+      if(read<T>(ai,j) != read<T>(bi,j))
+        return false;
+    }
+  }
+  return true;
+}
+
+template<class T>
+void array2Equals(vm::stack *s)
+{
+  s->push(Array2Equals<T>(s));
+}
+
+template<class T>
+void array2NotEquals(vm::stack *s)
+{
+  s->push(!Array2Equals<T>(s));
+}
+
 template<class T>
 struct compare {
   bool operator() (const vm::item& a, const vm::item& b)
@@ -218,7 +253,7 @@ void searchArray(vm::stack *s)
   Int u=size-1;
   if(key >= read<T>(a,u)) {s->push(u); return;}
   Int l=0;
-	
+        
   while (l < u) {
     Int i=(l+u)/2;
     if(read<T>(a,i) <= key && key < read<T>(a,i+1)) {s->push(i); return;}
@@ -257,7 +292,7 @@ void write(vm::stack *s)
       suffix->call(s);
     } else if(defaultfile) {
       try {
-	f->writeline();
+        f->writeline();
       } catch (quit&) {
       }
     }
@@ -287,30 +322,30 @@ void writeArray(vm::stack *s)
       cont=false;
       bool first=true;
       if(i < asize) {
-	vm::item& I=(*a)[i];
-	if(defaultfile) cout << i << ":\t";
-	if(!I.empty())
-	  f->write(vm::get<T>(I));
-	cont=true;
-	first=false;
+        vm::item& I=(*a)[i];
+        if(defaultfile) cout << i << ":\t";
+        if(!I.empty())
+          f->write(vm::get<T>(I));
+        cont=true;
+        first=false;
       }
       unsigned count=0;
       for(size_t j=0; j < Asize; ++j) {
-	array *Aj=read<array*>(A,j);
-	size_t Ajsize=checkArray(Aj);
-	if(i < Ajsize) {
-	  if(f->text()) {
-	    if(first && defaultfile) cout << i << ":\t";
-	    for(unsigned k=0; k <= count; ++k)
-	      f->write(tab);
-	    count=0;
-	  }
-	  vm::item& I=(*Aj)[i];
-	  if(!I.empty())
-	    f->write(vm::get<T>(I));
-	  cont=true;
-	  first=false;
-	} else count++;
+        array *Aj=read<array*>(A,j);
+        size_t Ajsize=checkArray(Aj);
+        if(i < Ajsize) {
+          if(f->text()) {
+            if(first && defaultfile) cout << i << ":\t";
+            for(unsigned k=0; k <= count; ++k)
+              f->write(tab);
+            count=0;
+          }
+          vm::item& I=(*Aj)[i];
+          if(!I.empty())
+            f->write(vm::get<T>(I));
+          cont=true;
+          first=false;
+        } else count++;
       }
       ++i;
       if(cont && f->text()) f->writeline();
@@ -331,20 +366,20 @@ void writeArray2(vm::stack *s)
   else if(!f->isOpen()) return;
   
   try {
-  for(size_t i=0; i < size; i++) {
-    vm::item& I=(*a)[i];
-    if(!I.empty()) {
-      array *ai=vm::get<array*>(I);
-      size_t aisize=checkArray(ai);
-      for(size_t j=0; j < aisize; j++) {
-	if(j > 0 && f->text()) f->write(tab);
-	vm::item& I=(*ai)[j];
-	if(!I.empty())
-	  f->write(vm::get<T>(I));
+    for(size_t i=0; i < size; i++) {
+      vm::item& I=(*a)[i];
+      if(!I.empty()) {
+        array *ai=vm::get<array*>(I);
+        size_t aisize=checkArray(ai);
+        for(size_t j=0; j < aisize; j++) {
+          if(j > 0 && f->text()) f->write(tab);
+          vm::item& I=(*ai)[j];
+          if(!I.empty())
+            f->write(vm::get<T>(I));
+        }
       }
+      if(f->text()) f->writeline();
     }
-    if(f->text()) f->writeline();
-  }
   } catch (quit&) {
   }
   f->flush();
@@ -361,29 +396,29 @@ void writeArray3(vm::stack *s)
   else if(!f->isOpen()) return;
   
   try {
-  for(size_t i=0; i < size;) {
-    vm::item& I=(*a)[i];
-    if(!I.empty()) {
-      array *ai=vm::get<array*>(I);
-      size_t aisize=checkArray(ai);
-      for(size_t j=0; j < aisize; j++) {
-	vm::item& I=(*ai)[j];
-	if(!I.empty()) {
-	  array *aij=vm::get<array*>(I);
-	  size_t aijsize=checkArray(aij);
-	  for(size_t k=0; k < aijsize; k++) {
-	    if(k > 0 && f->text()) f->write(tab);
-	    vm::item& I=(*aij)[k];
-	    if(!I.empty())
-	      f->write(vm::get<T>(I));
-	  }
-	}
-	if(f->text()) f->writeline();
+    for(size_t i=0; i < size;) {
+      vm::item& I=(*a)[i];
+      if(!I.empty()) {
+        array *ai=vm::get<array*>(I);
+        size_t aisize=checkArray(ai);
+        for(size_t j=0; j < aisize; j++) {
+          vm::item& I=(*ai)[j];
+          if(!I.empty()) {
+            array *aij=vm::get<array*>(I);
+            size_t aijsize=checkArray(aij);
+            for(size_t k=0; k < aijsize; k++) {
+              if(k > 0 && f->text()) f->write(tab);
+              vm::item& I=(*aij)[k];
+              if(!I.empty())
+                f->write(vm::get<T>(I));
+            }
+          }
+          if(f->text()) f->writeline();
+        }
       }
+      ++i;
+      if(i < size && f->text()) f->writeline();
     }
-    ++i;
-    if(i < size && f->text()) f->writeline();
-  }
   } catch (quit&) {
   }
   f->flush();
@@ -400,6 +435,10 @@ void arrayFunc(vm::stack *s)
   s->push(c);
 }
 
+vm::array *Identity(Int n);
+camp::triple operator *(const vm::array& a, const camp::triple& v);
+camp::triple multshiftless(const vm::array& t, const camp::triple& v);
+
 } // namespace run
 
 #endif // ARRAYOP_H
