@@ -25,9 +25,8 @@ public:
   bool endclip() {return true;}
   
   void bounds(bbox& b, iopipestream&, boxvector&, bboxlist& bboxstack) {
-    if(bboxstack.size() < 2) {
+    if(bboxstack.size() < 2)
       reportError("endclip without matching beginclip");
-    }
     b.clip(bboxstack.back());
     bboxstack.pop_back();
     b += bboxstack.back();
@@ -47,7 +46,7 @@ public:
   }
 
   bool write(texfile *out, const bbox& bpath) {
-    out->verbatimline("\\end{picture}%");
+    out->verbatimline(settings::endpicture(out->texengine));
     out->verbatim("\\kern");
     double width=bpath.right-bpath.left;
     out->write(-width*ps2tex);
