@@ -12,9 +12,12 @@
 
 #include "common.h"
 #include "bbox.h"
+#include "bbox3.h"
 #include "pen.h"
 #include "psfile.h"
 #include "texfile.h"
+#include "prcfile.h"
+#include "glrender.h"
 
 namespace camp {
 
@@ -35,10 +38,10 @@ public:
       pair B=p[i < 3 ? i+1 : 0];
       double de=(b.x-a.x)*(A.y-B.y)-(A.x-B.x)*(b.y-a.y);
       if(de != 0.0) {
-	de=1.0/de;
-	double t=((A.x-a.x)*(A.y-B.y)-(A.x-B.x)*(A.y-a.y))*de;
-	double T=((b.x-a.x)*(A.y-a.y)-(A.x-a.x)*(b.y-a.y))*de;
-	if(0 <= t && t <= 1 && 0 <= T && T <= 1) return true;
+        de=1.0/de;
+        double t=((A.x-a.x)*(A.y-B.y)-(A.x-B.x)*(A.y-a.y))*de;
+        double T=((b.x-a.x)*(A.y-a.y)-(A.x-a.x)*(b.y-a.y))*de;
+        if(0 <= t && t <= 1 && 0 <= T && T <= 1) return true;
       }
     }
     return false;
@@ -62,7 +65,7 @@ public:
       pair pi=p[i];
       pair pj=p[i < 3 ? i+1 : 0];
       if(((pi.y <= z.y && z.y < pj.y) || (pj.y <= z.y && z.y < pi.y)) &&
-	 z.x < pi.x+(pj.x-pi.x)*(z.y-pi.y)/(pj.y-pi.y)) c=!c;
+         z.x < pi.x+(pj.x-pi.x)*(z.y-pi.y)/(pj.y-pi.y)) c=!c;
     }
     return c;
   }
@@ -99,11 +102,19 @@ public:
   // Adjust the bbox of the picture based on the addition of this
   // element. The iopipestream is needed for determining label sizes.
   virtual void bounds(bbox&, iopipestream&, boxvector&, bboxlist&) {}
+  virtual void bounds(bbox3&) {}
+
+  virtual void bounds(pair &b, double (*m)(double, double),
+                      double (*x)(const triple&, double*),
+                      double (*y)(const triple&, double*),
+                      double *t, bool &first){}
 
   virtual bool islabel() {return false;}
-
+  
   virtual bool islayer() {return false;}
 
+  virtual bool is3D() {return false;}
+  
   virtual bool endclip() {return false;}
   
   virtual bool begingroup() {return false;}
@@ -112,20 +123,37 @@ public:
 
   virtual void save(bool b) {}
   
-  // Handles its output in a PostScript file
+  // Output to a PostScript file
   virtual bool draw(psfile *) {
-    return true;
+    return false;
   }
 
-  // Handles its output in a TeX file
+  // Output to a TeX file
   virtual bool write(texfile *, const bbox&) {
-    return true;
+    return false;
   }
 
+  // Output to a PRC file
+  virtual bool write(prcfile *) {
+    return false;
+  }
+
+  // Used to compute deviation of a surface from a quadrilateral.
+  virtual void displacement() {}
+
+  // Render with OpenGL
+  virtual void render(GLUnurbs *nurb, double size2, 
+                      const triple& Min, const triple& Max,
+                      double perspective, bool transparent) {}
+
   // Transform as part of a picture.
   virtual drawElement *transformed(const transform&) {
     return this;
   }
+  
+  virtual drawElement *transformed(const vm::array&) {
+    return this;
+  }
 };
 
 // Base class for drawElements that involve paths.
@@ -181,6 +209,8 @@ public:
     return p.cyclic();
   }
   
+  void strokebounds(bbox& b, const path& p);
+    
   virtual void penSave(psfile *out)
   {
     if (!pentype.getTransform().isIdentity())
@@ -246,6 +276,20 @@ public:
     b += bpath;
   }
   
+  void strokepath(psfile *out) {
+    // strokepath and evenodd are incompatible
+    static pen zerowinding=pen((FillRule) ZEROWINDING);
+    pentype=pentype+zerowinding;
+    out->setpen(pentype);
+    out->strokepath();
+  }
+  
+  void strokebounds(bbox& b) {
+    for(size_t i=0; i < size; i++)
+      drawPathPenBase::strokebounds(bpath,vm::read<path>(P,i));
+    b += bpath;
+  }
+  
   void writepath(psfile *out) {
     for(size_t i=0; i < size; i++) 
       out->write(vm::read<path>(P,i),i == 0);
