@@ -7,7 +7,6 @@
 
 #include <cstdio>
 #include <cstdlib>
-#include <cstdarg>
 
 #include "errormsg.h"
 
@@ -70,6 +69,17 @@ void errorstream::warning(position pos)
   anyWarnings = true;
 }
 
+void errorstream::disable(string s)
+{
+  out << ": to ignore, use nowarn(\""+s+"\");";
+}
+
+void errorstream::fatal(position pos)
+{
+  message(pos,"abort: ");
+  anyErrors = true;
+}
+
 void errorstream::trace(position pos)
 {
   static position lastpos;
@@ -91,3 +101,8 @@ void errorstream::sync()
   floating = false;
 }
 
+void outOfMemory() 
+{
+  cerr << "error: out of memory" << endl;
+  exit(1);
+}
