@@ -118,7 +118,7 @@ class errorstream {
   ostream& out;
   bool anyErrors;
   bool anyWarnings;
-  bool floating;	// Was a message output without a terminating newline?
+  bool floating;        // Was a message output without a terminating newline?
   
   // Is there an error that warrants the asy process to return 1 instead of 0?
   bool anyStatusErrors;
@@ -155,7 +155,13 @@ public:
 
   // Indicate potential problems in the code, but the code is still usable.
   void warning(position pos);
+  
+  // Indicate how a warning may be disabled.
+  void disable(string s);
 
+  // Single a fatal error and execute the main process.
+  void fatal(position pos);
+  
   // Print out position in code to aid debugging.
   void trace(position pos);
   
@@ -194,6 +200,7 @@ public:
 };
 
 extern errorstream em;
+void outOfMemory();
 
 GC_DECLARE_PTRFREE(nullPosInitializer);
 
