@@ -15,12 +15,17 @@
 
 namespace camp {
 
+static const double tex2ps=72.0/72.27;
+static const double ps2tex=1.0/tex2ps;
+  
 static const string DEFPAT="<default>";
 static const string DEFLATEXFONT="\\usefont{\\ASYencoding}{\\ASYfamily}{\\ASYseries}{\\ASYshape}";
-static const string DEFTEXFONT="\\font\\ASYfont=cmr12\\ASYfont";
+static const string DEFCONTEXTFONT="modern";
+static const string DEFTEXFONT="cmr12";
 static const double DEFWIDTH=-1;
 static const Int DEFCAP=-1;
 static const Int DEFJOIN=-1;
+static const double DEFMITER=0;
   
 static const struct invisiblepen_t {} invisiblepen={};
 static const struct setlinewidth_t {} setlinewidth={};
@@ -29,6 +34,7 @@ static const struct setfontsize_t {} setfontsize={};
 static const struct setpattern_t {} setpattern={};
 static const struct setlinecap_t {} setlinecap={};
 static const struct setlinejoin_t {} setlinejoin={};
+static const struct setmiterlimit_t {} setmiterlimit={};
 static const struct setoverwrite_t {} setoverwrite={};
 static const struct initialpen_t {} initialpen={};
 static const struct resolvepen_t {} resolvepen={};
@@ -40,7 +46,7 @@ const Int nJoin=sizeof(Join)/sizeof(string);
   
 enum overwrite_t {DEFWRITE=-1,ALLOW,SUPPRESS,SUPPRESSQUIET,MOVE,MOVEQUIET};
 static const string OverwriteTag[]={"Allow","Suppress","SupressQuiet",
-					 "Move","MoveQuiet"};
+                                    "Move","MoveQuiet"};
 const Int nOverwrite=sizeof(OverwriteTag)/sizeof(string);
   
 enum FillRule {DEFFILL=-1,ZEROWINDING,EVENODD};
@@ -53,7 +59,7 @@ static const string BaseLineTag[]={"NoAlign","Align"};
 const Int nBaseLine=sizeof(BaseLineTag)/sizeof(string);
   
 enum ColorSpace {DEFCOLOR=0,INVISIBLE,GRAYSCALE,RGB,CMYK,PATTERN};
-extern const unsigned ColorComponents[];
+extern const size_t ColorComponents[];
 static const string ColorDeviceSuffix[]={"","","Gray","RGB","CMYK",""};
 const unsigned nColorSpace=sizeof(ColorDeviceSuffix)/sizeof(string);
   
@@ -62,10 +68,10 @@ const unsigned nColorSpace=sizeof(ColorDeviceSuffix)/sizeof(string);
 class LineType
 {
 public:  
-  string pattern;	// The string for the PostScript style line pattern.
+  string pattern;       // The string for the PostScript style line pattern.
   double offset;        // The offset in the pattern at which to start drawing.
-  bool scale;		// Scale the line type values by the pen width?
-  bool adjust;		// Adjust the line type values to fit the arclength?
+  bool scale;           // Scale the line type values by the pen width?
+  bool adjust;          // Adjust the line type values to fit the arclength?
   
   LineType(string pattern, double offset, bool scale, bool adjust) : 
     pattern(pattern), offset(offset), scale(scale), adjust(adjust) {}
@@ -94,10 +100,10 @@ inline bool operator == (Transparency a, Transparency b) {
 }
   
 static const string BlendMode[]={"Compatible","Normal","Multiply","Screen",
-				      "Overlay","SoftLight","HardLight",
-				      "ColorDodge","ColorBurn","Darken",
-				      "Lighten","Difference","Exclusion",
-				      "Hue","Saturation","Color","Luminosity"};
+                                 "Overlay","SoftLight","HardLight",
+                                 "ColorDodge","ColorBurn","Darken",
+                                 "Lighten","Difference","Exclusion",
+                                 "Hue","Saturation","Color","Luminosity"};
 const Int nBlendMode=sizeof(BlendMode)/sizeof(string);
 
 static const transform nullTransform=transform(0.0,0.0,0.0,0.0,0.0,0.0);  
@@ -117,15 +123,16 @@ class pen : public gc {
   double lineskip;  
   
   ColorSpace color;
-  double r,g,b; 	// RGB or CMY value
-  double grey; 		// grayscale or K value
+  double r,g,b;         // RGB or CMY value
+  double grey;          // grayscale or K value
   
-  string pattern;	// The name of the user-defined fill/draw pattern
-  FillRule fillrule; 	// Zero winding-number (default) or even-odd rule
-  BaseLine baseline;	// Align to TeX baseline?
+  string pattern;       // The name of the user-defined fill/draw pattern
+  FillRule fillrule;    // Zero winding-number (default) or even-odd rule
+  BaseLine baseline;    // Align to TeX baseline?
   Transparency transparency;
   Int linecap;
   Int linejoin;
+  double miterlimit;
   overwrite_t overwrite;
   
   // The transformation applied to the pen nib for calligraphic effects.
@@ -169,20 +176,23 @@ public:
     r(0), g(0), b(0), grey(0),
     pattern(DEFPAT), fillrule(DEFFILL), baseline(DEFBASE),
     transparency(DEFTRANSP),
-    linecap(DEFCAP), linejoin(DEFJOIN), overwrite(DEFWRITE), t(nullTransform) {}
+    linecap(DEFCAP), linejoin(DEFJOIN), miterlimit(DEFMITER),
+    overwrite(DEFWRITE), t(nullTransform) {}
 
   pen(const LineType& line, double linewidth, const path& P,
       const string& font, double fontsize, double lineskip,
       ColorSpace color, double r, double g, double b,  double grey,
       const string& pattern, FillRule fillrule, BaseLine baseline,
       const Transparency& transparency,
-      Int linecap, Int linejoin, overwrite_t overwrite, const transform& t) :
+      Int linecap, Int linejoin, double miterlimit,
+      overwrite_t overwrite, const transform& t) :
     line(line), linewidth(linewidth), P(P),
     font(font), fontsize(fontsize), lineskip(lineskip), color(color),
     r(r), g(g), b(b), grey(grey),
     pattern(pattern), fillrule(fillrule), baseline(baseline),
     transparency(transparency),
-    linecap(linecap), linejoin(linejoin), overwrite(overwrite), t(t) {}
+    linecap(linecap), linejoin(linejoin), miterlimit(miterlimit),
+    overwrite(overwrite), t(t) {}
       
   pen(invisiblepen_t) : 
     line(DEFLINE), linewidth(DEFWIDTH), P(nullpath),
@@ -190,7 +200,8 @@ public:
     r(0), g(0), b(0), grey(0),
     pattern(DEFPAT), fillrule(DEFFILL), baseline(DEFBASE),
     transparency(DEFTRANSP),
-    linecap(DEFCAP), linejoin(DEFJOIN), overwrite(DEFWRITE), t(nullTransform) {}
+    linecap(DEFCAP), linejoin(DEFJOIN), miterlimit(DEFMITER),
+    overwrite(DEFWRITE), t(nullTransform) {}
   
   pen(setlinewidth_t, double linewidth) : 
     line(DEFLINE), linewidth(linewidth), P(nullpath),
@@ -198,7 +209,8 @@ public:
     r(0), g(0), b(0), grey(0),
     pattern(DEFPAT), fillrule(DEFFILL), baseline(DEFBASE),
     transparency(DEFTRANSP),
-    linecap(DEFCAP), linejoin(DEFJOIN), overwrite(DEFWRITE), t(nullTransform) {}
+    linecap(DEFCAP), linejoin(DEFJOIN), miterlimit(DEFMITER),
+    overwrite(DEFWRITE), t(nullTransform) {}
   
   pen(path P) : 
     line(DEFLINE), linewidth(DEFWIDTH), P(P),
@@ -206,7 +218,8 @@ public:
     r(0), g(0), b(0), grey(0),
     pattern(DEFPAT), fillrule(DEFFILL), baseline(DEFBASE),
     transparency(DEFTRANSP),
-    linecap(DEFCAP), linejoin(DEFJOIN), overwrite(DEFWRITE), t(nullTransform) {}
+    linecap(DEFCAP), linejoin(DEFJOIN), miterlimit(DEFMITER),
+    overwrite(DEFWRITE), t(nullTransform) {}
   
   pen(const LineType& line) :
     line(line), linewidth(DEFWIDTH), P(nullpath),
@@ -214,7 +227,8 @@ public:
     r(0), g(0), b(0), grey(0),
     pattern(DEFPAT), fillrule(DEFFILL), baseline(DEFBASE),
     transparency(DEFTRANSP),
-    linecap(DEFCAP), linejoin(DEFJOIN), overwrite(DEFWRITE), t(nullTransform) {}
+    linecap(DEFCAP), linejoin(DEFJOIN), miterlimit(DEFMITER),
+    overwrite(DEFWRITE), t(nullTransform) {}
   
   pen(setfont_t, string font) :
     line(DEFLINE), linewidth(DEFWIDTH), P(nullpath),
@@ -222,7 +236,8 @@ public:
     r(0), g(0), b(0), grey(0),
     pattern(DEFPAT), fillrule(DEFFILL), baseline(DEFBASE),
     transparency(DEFTRANSP),
-    linecap(DEFCAP), linejoin(DEFJOIN), overwrite(DEFWRITE), t(nullTransform) {}
+    linecap(DEFCAP), linejoin(DEFJOIN), miterlimit(DEFMITER),
+    overwrite(DEFWRITE), t(nullTransform) {}
   
   pen(setfontsize_t, double fontsize, double lineskip) :
     line(DEFLINE), linewidth(DEFWIDTH), P(nullpath),
@@ -230,7 +245,8 @@ public:
     r(0), g(0), b(0), grey(0),
     pattern(DEFPAT), fillrule(DEFFILL), baseline(DEFBASE),
     transparency(DEFTRANSP),
-    linecap(DEFCAP), linejoin(DEFJOIN), overwrite(DEFWRITE), t(nullTransform) {}
+    linecap(DEFCAP), linejoin(DEFJOIN), miterlimit(DEFMITER),
+    overwrite(DEFWRITE), t(nullTransform) {}
   
   pen(setpattern_t, const string& pattern) :
     line(DEFLINE), linewidth(DEFWIDTH), P(nullpath),
@@ -238,7 +254,8 @@ public:
     r(0), g(0), b(0), grey(0),
     pattern(pattern), fillrule(DEFFILL), baseline(DEFBASE),
     transparency(DEFTRANSP),
-    linecap(DEFCAP), linejoin(DEFJOIN), overwrite(DEFWRITE), t(nullTransform) {}
+    linecap(DEFCAP), linejoin(DEFJOIN), miterlimit(DEFMITER),
+    overwrite(DEFWRITE), t(nullTransform) {}
   
   pen(FillRule fillrule) :
     line(DEFLINE), linewidth(DEFWIDTH), P(nullpath),
@@ -246,7 +263,8 @@ public:
     r(0), g(0), b(0), grey(0),
     pattern(DEFPAT), fillrule(fillrule), baseline(DEFBASE),
     transparency(DEFTRANSP),
-    linecap(DEFCAP), linejoin(DEFJOIN), overwrite(DEFWRITE), t(nullTransform) {}
+    linecap(DEFCAP), linejoin(DEFJOIN), miterlimit(DEFMITER),
+    overwrite(DEFWRITE), t(nullTransform) {}
   
   pen(BaseLine baseline) :
     line(DEFLINE), linewidth(DEFWIDTH), P(nullpath),
@@ -254,7 +272,8 @@ public:
     r(0), g(0), b(0), grey(0),
     pattern(DEFPAT), fillrule(DEFFILL), baseline(baseline),
     transparency(DEFTRANSP),
-    linecap(DEFCAP), linejoin(DEFJOIN), overwrite(DEFWRITE), t(nullTransform) {}
+    linecap(DEFCAP), linejoin(DEFJOIN), miterlimit(DEFMITER),
+    overwrite(DEFWRITE), t(nullTransform) {}
   
   pen(const Transparency& transparency) :
     line(DEFLINE), linewidth(DEFWIDTH), P(nullpath),
@@ -262,7 +281,8 @@ public:
     r(0), g(0), b(0), grey(0),
     pattern(DEFPAT), fillrule(DEFFILL), baseline(DEFBASE),
     transparency(transparency),
-    linecap(DEFCAP), linejoin(DEFJOIN), overwrite(DEFWRITE), t(nullTransform) {}
+    linecap(DEFCAP), linejoin(DEFJOIN), miterlimit(DEFMITER),
+    overwrite(DEFWRITE), t(nullTransform) {}
   
   pen(setlinecap_t, Int linecap) :
     line(DEFLINE), linewidth(DEFWIDTH), P(nullpath),
@@ -270,7 +290,8 @@ public:
     r(0), g(0), b(0), grey(0),
     pattern(DEFPAT), fillrule(DEFFILL), baseline(DEFBASE),
     transparency(DEFTRANSP),
-    linecap(linecap), linejoin(DEFJOIN), overwrite(DEFWRITE), t(nullTransform) {}
+    linecap(linecap), linejoin(DEFJOIN), miterlimit(DEFMITER),
+    overwrite(DEFWRITE), t(nullTransform) {}
   
   pen(setlinejoin_t, Int linejoin) :
     line(DEFLINE), linewidth(DEFWIDTH), P(nullpath),
@@ -278,7 +299,17 @@ public:
     r(0), g(0), b(0), grey(0),
     pattern(DEFPAT), fillrule(DEFFILL), baseline(DEFBASE),
     transparency(DEFTRANSP),
-    linecap(DEFCAP), linejoin(linejoin), overwrite(DEFWRITE), t(nullTransform) {}
+    linecap(DEFCAP), linejoin(linejoin), miterlimit(DEFMITER),
+    overwrite(DEFWRITE), t(nullTransform) {}
+  
+  pen(setmiterlimit_t, double miterlimit) :
+    line(DEFLINE), linewidth(DEFWIDTH), P(nullpath),
+    font(""), fontsize(0.0), lineskip(0.0), color(DEFCOLOR),
+    r(0), g(0), b(0), grey(0),
+    pattern(DEFPAT), fillrule(DEFFILL), baseline(DEFBASE),
+    transparency(DEFTRANSP),
+    linecap(DEFCAP), linejoin(DEFJOIN), miterlimit(miterlimit),
+    overwrite(DEFWRITE), t(nullTransform) {}
   
   pen(setoverwrite_t, overwrite_t overwrite) :
     line(DEFLINE), linewidth(DEFWIDTH), P(nullpath),
@@ -286,7 +317,8 @@ public:
     r(0), g(0), b(0), grey(0),
     pattern(DEFPAT), fillrule(DEFFILL), baseline(DEFBASE),
     transparency(DEFTRANSP),
-    linecap(DEFCAP), linejoin(DEFJOIN), overwrite(overwrite), t(nullTransform) {}
+    linecap(DEFCAP), linejoin(DEFJOIN), miterlimit(DEFMITER),
+    overwrite(overwrite), t(nullTransform) {}
   
   explicit pen(double grey) :
     line(DEFLINE), linewidth(DEFWIDTH), P(nullpath),
@@ -294,7 +326,8 @@ public:
     r(0.0), g(0.0), b(0.0), grey(pos0(grey)),
     pattern(DEFPAT), fillrule(DEFFILL), baseline(DEFBASE),
     transparency(DEFTRANSP),
-    linecap(DEFCAP), linejoin(DEFJOIN), overwrite(DEFWRITE), t(nullTransform)
+    linecap(DEFCAP), linejoin(DEFJOIN), miterlimit(DEFMITER),
+    overwrite(DEFWRITE), t(nullTransform)
   {greyrange();}
   
   pen(double r, double g, double b) : 
@@ -303,7 +336,8 @@ public:
     r(pos0(r)), g(pos0(g)), b(pos0(b)),  grey(0.0), 
     pattern(DEFPAT), fillrule(DEFFILL), baseline(DEFBASE),
     transparency(DEFTRANSP),
-    linecap(DEFCAP), linejoin(DEFJOIN), overwrite(DEFWRITE), t(nullTransform)
+    linecap(DEFCAP), linejoin(DEFJOIN), miterlimit(DEFMITER),
+    overwrite(DEFWRITE), t(nullTransform)
   {rgbrange();}
   
   pen(double c, double m, double y, double k) :
@@ -312,7 +346,8 @@ public:
     r(pos0(c)), g(pos0(m)), b(pos0(y)), grey(pos0(k)),
     pattern(DEFPAT), fillrule(DEFFILL), baseline(DEFBASE),
     transparency(DEFTRANSP),
-    linecap(DEFCAP), linejoin(DEFJOIN), overwrite(DEFWRITE), t(nullTransform)
+    linecap(DEFCAP), linejoin(DEFJOIN), miterlimit(DEFMITER),
+    overwrite(DEFWRITE), t(nullTransform)
   {cmykrange();}
   
   // Construct one pen from another, resolving defaults
@@ -324,14 +359,14 @@ public:
     r(p.red()), g(p.green()), b(p.blue()), grey(p.gray()),
     pattern(""), fillrule(p.Fillrule()), baseline(p.Baseline()),
     transparency(Transparency(p.blend(), p.opacity())),
-    linecap(p.cap()), linejoin(p.join()), overwrite(p.Overwrite()),
-    t(p.getTransform()) {}
+    linecap(p.cap()), linejoin(p.join()), miterlimit(p.miter()),
+    overwrite(p.Overwrite()), t(p.getTransform()) {}
   
   static pen initialpen() {
     return pen(LineType("",0,true,true),0.5,nullpath,"",12.0,12.0*1.2,
-	       GRAYSCALE,
-	       0.0,0.0,0.0,0.0,"",ZEROWINDING,NOBASEALIGN,
-	       DEFTRANSP,1,1,ALLOW,identity);
+               GRAYSCALE,
+               0.0,0.0,0.0,0.0,"",ZEROWINDING,NOBASEALIGN,
+               DEFTRANSP,1,1,10.0,ALLOW,identity);
   }
   
   pen(initialpen_t) : 
@@ -339,8 +374,8 @@ public:
     font("<invalid>"), fontsize(-1.0), lineskip(-1.0), color(INVISIBLE),
     r(0.0), g(0.0), b(0.0), grey(0.0),
     pattern(DEFPAT), fillrule(DEFFILL), baseline(NOBASEALIGN),
-    transparency(DEFTRANSP),linecap(-2), linejoin(-2), overwrite(DEFWRITE),
-    t(nullTransform) {}
+    transparency(DEFTRANSP),linecap(-2), linejoin(-2), miterlimit(-1.0),
+    overwrite(DEFWRITE), t(nullTransform) {}
   
   double width() const {
     return linewidth == DEFWIDTH ? defaultpen().linewidth : linewidth;
@@ -350,20 +385,36 @@ public:
     return P.empty() ? defaultpen().P : P;
   }
   
+  double size() const {
+    return fontsize == 0.0 ? defaultpen().fontsize : fontsize;
+  }
+  
   string Font() const {
     if(font.empty()) {
-      if(defaultpen().font.empty())
-	return settings::latex(settings::getSetting<string>("tex")) ? 
-	  DEFLATEXFONT : DEFTEXFONT;
+      if(defaultpen().font.empty()) {
+        string texengine=settings::getSetting<string>("tex");
+        if(settings::latex(texengine))
+          return DEFLATEXFONT;
+        else if(texengine == "none")
+          return settings::getSetting<string>("textinitialfont");
+        else {
+          ostringstream buf;
+  // Work around misalignment in ConTeXt switchtobodyfont if font is not found.
+          if(texengine == "context")
+            buf << "\\switchtobodyfont[" 
+                << DEFCONTEXTFONT << "," << size()*ps2tex 
+                << "pt]\\removeunwantedspaces%" << newl;
+          else
+            buf << "\\font\\ASYfont=" << DEFTEXFONT
+              << " at " << size()*ps2tex << "pt\\ASYfont";
+          return buf.str();
+        }
+      }
       else return defaultpen().font;
     }
     return font;
   }
   
-  double size() const {
-    return fontsize == 0.0 ? defaultpen().fontsize : fontsize;
-  }
-  
   double Lineskip() const {
     return lineskip == 0.0 ? defaultpen().lineskip : lineskip;
   }
@@ -388,7 +439,7 @@ public:
   }
   
   bool evenodd() const {
-    return fillrule == EVENODD;
+    return Fillrule() == EVENODD;
   }
   
   bool inside(Int count) const {
@@ -421,6 +472,10 @@ public:
     return linejoin == DEFJOIN ? defaultpen().linejoin : linejoin;
   }
   
+  double miter() const {
+    return miterlimit == DEFMITER ? defaultpen().miterlimit : miterlimit;
+  }
+  
   overwrite_t Overwrite() const {
     return overwrite == DEFWRITE ? defaultpen().overwrite : overwrite;
   }
@@ -475,15 +530,12 @@ public:
   
   void rgbtocmyk() {
     double sat=rgbsaturation();
-    r=1.0-r;
-    g=1.0-g;
-    b=1.0-b;
     grey=1.0-sat;
     if(sat) {
       double scale=1.0/sat;
-      r=(r-grey)*scale;
-      g=(g-grey)*scale;
-      b=(b-grey)*scale;
+      r=1.0-r*scale;
+      g=1.0-g*scale;
+      b=1.0-b*scale;
     }
     color=CMYK;
   }
@@ -502,6 +554,25 @@ public:
     rgbtogrey();
   }
   
+  void togrey() {
+    if(rgb()) rgbtogrey();
+    else if(cmyk()) cmyktogrey();
+  }
+  
+  void torgb() {
+    if(cmyk()) cmyktorgb();
+    else if(grayscale()) greytorgb();
+  }
+  
+  void tocmyk() {
+    if(rgb()) rgbtocmyk();
+    else if(grayscale()) greytocmyk();
+  }
+  
+  void settransparency(const pen& p) {
+    transparency=p.transparency;
+  }
+                                                               
   void convert() {
     if(settings::gray || settings::bw) {
       if(rgb()) rgbtogrey();
@@ -517,29 +588,29 @@ public:
     if(color == c) return true;
     
     switch(color) {
-    case PATTERN:
-    case INVISIBLE:
-      break;
-    case DEFCOLOR:
+      case PATTERN:
+      case INVISIBLE:
+        break;
+      case DEFCOLOR:
       {
-	return true;
-	break;
+        return true;
+        break;
       }
       break;
-    case GRAYSCALE:
+      case GRAYSCALE:
       {
-	if(c == RGB) {greytorgb(); return true;}
-	else if(c == CMYK) {greytocmyk(); return true;}
-	break;
+        if(c == RGB) {greytorgb(); return true;}
+        else if(c == CMYK) {greytocmyk(); return true;}
+        break;
       }
-    case RGB:
+      case RGB:
       {
-	if(c == CMYK) {rgbtocmyk(); return true;}
-	break;
+        if(c == CMYK) {rgbtocmyk(); return true;}
+        break;
       }
-    case CMYK:
+      case CMYK:
       {
-	break;
+        break;
       }
     }
     return false;
@@ -549,123 +620,138 @@ public:
     pen p=q;
     if(x < 0.0) x = 0.0;
     switch(p.color) {
-    case PATTERN:
-    case INVISIBLE:
-    case DEFCOLOR:
-      break;
-    case GRAYSCALE:
+      case PATTERN:
+      case INVISIBLE:
+      case DEFCOLOR:
+        break;
+      case GRAYSCALE:
       {
-	p.grey *= x;
-	p.greyrange();
-	break;
+        p.grey *= x;
+        p.greyrange();
+        break;
       }
-    case RGB:
+      case RGB:
       {
-	p.r *= x;
-	p.g *= x;
-	p.b *= x;
-	p.rgbrange();
-	break;
+        p.r *= x;
+        p.g *= x;
+        p.b *= x;
+        p.rgbrange();
+        break;
       }
-    case CMYK:
+      case CMYK:
       {
-	p.r *= x;
-	p.g *= x;
-	p.b *= x;
-	p.grey *= x;
-	p.cmykrange();
-	break;
+        p.r *= x;
+        p.g *= x;
+        p.b *= x;
+        p.grey *= x;
+        p.cmykrange();
+        break;
       }
     }
     return p;
   }
   
   friend pen operator + (const pen& p, const pen& q) {
-    double R=0.0,G=0.0,B=0.0,greyval=0.0;
     pen P=p;
     pen Q=q;
     
     if(P.color == PATTERN && P.pattern.empty()) P.color=DEFCOLOR;
     ColorSpace colorspace=(ColorSpace) max((Int) P.color,(Int) Q.color);
     
-  switch(colorspace) {
-    case PATTERN:
-    case INVISIBLE:
-    case DEFCOLOR:
-      break;
-    case GRAYSCALE:
+    if(!(p.transparency == DEFTRANSP && q.transparency == DEFTRANSP))
+      P.transparency.opacity=max(p.opacity(),q.opacity());
+    
+    switch(colorspace) {
+      case PATTERN:
+      case INVISIBLE:
+      case DEFCOLOR:
+        break;
+      case GRAYSCALE:
       {
-	if(p.color != GRAYSCALE) greyval=q.grey;
-	else {
-	  if(q.color != GRAYSCALE) greyval=p.grey;
-	  else greyval=0.5*(p.grey+q.grey);
-	}
-	break;
+        P.grey += Q.grey;
+        P.greyrange();
+        break;
       }
       
-    case RGB:
+      case RGB:
       {
-	double sat;
-	if (P.color == GRAYSCALE) {
-	  P.greytorgb();
-	  sat=0.5*(P.rgbsaturation()+Q.rgbsaturation());
-	} else {
-	  if (Q.color == GRAYSCALE) {
-	    Q.greytorgb();
-	    sat=0.5*(P.rgbsaturation()+Q.rgbsaturation());
-	  } else sat=max(P.rgbsaturation(),Q.rgbsaturation());
-	}
-	// Mix colors
-	P.r += Q.r;
-	P.g += Q.g;
-	P.b += Q.b;
-	double newsat=P.rgbsaturation();
-	double scale=newsat ? sat/newsat: 1.0;
-	R=P.r*scale;
-	G=P.g*scale;
-	B=P.b*scale;
-	break;
+        if(P.color == GRAYSCALE) P.greytorgb();
+        else if(Q.color == GRAYSCALE) Q.greytorgb();
+        
+        P.r += Q.r;
+        P.g += Q.g;
+        P.b += Q.b;
+        P.rgbrange();
+        break;
       }
       
-    case CMYK:
+      case CMYK:
       {
-	if (P.color == GRAYSCALE) P.greytocmyk();
-	else if (Q.color == GRAYSCALE) Q.greytocmyk();
-	
-	if (P.color == RGB) P.rgbtocmyk();
-	else if (Q.color == RGB) Q.rgbtocmyk();
-	
-	double sat=max(P.cmyksaturation(),Q.cmyksaturation());      
-	// Mix colors
-	P.r += Q.r;
-	P.g += Q.g;
-	P.b += Q.b;
-	P.grey += Q.grey;
-	double newsat=P.cmyksaturation();
-	double scale=newsat ? sat/newsat: 1.0;
-	R=P.r*scale;
-	G=P.g*scale;
-	B=P.b*scale;
-	greyval=P.grey*scale;
-	break;
+        if(P.color == GRAYSCALE) P.greytocmyk();
+        else if(Q.color == GRAYSCALE) Q.greytocmyk();
+        
+        if(P.color == RGB) P.rgbtocmyk();
+        else if(Q.color == RGB) Q.rgbtocmyk();
+        
+        P.r += Q.r;
+        P.g += Q.g;
+        P.b += Q.b;
+        P.grey += Q.grey;
+        P.cmykrange();
+        break;
       }
     }
     
     return pen(q.line == DEFLINE ? p.line : q.line,
-	       q.linewidth == DEFWIDTH ? p.linewidth : q.linewidth,
-	       q.P.empty() ? p.P : q.P,
-	       q.font.empty() ? p.font : q.font,
-	       q.fontsize == 0.0 ? p.fontsize : q.fontsize,
-	       q.lineskip == 0.0 ? p.lineskip : q.lineskip,
-	       colorspace,R,G,B,greyval,
-	       q.pattern == DEFPAT ? p.pattern : q.pattern,
-	       q.fillrule == DEFFILL ? p.fillrule : q.fillrule,
-	       q.baseline == DEFBASE ? p.baseline : q.baseline,
-	       q.transparency == DEFTRANSP ? p.transparency : q.transparency,
-	       q.linecap == DEFCAP ? p.linecap : q.linecap,
-	       q.linejoin == DEFJOIN ? p.linejoin : q.linejoin,
-	       q.overwrite == DEFWRITE ? p.overwrite : q.overwrite,
-	       q.t.isNull() ? p.t : q.t);
+               q.linewidth == DEFWIDTH ? p.linewidth : q.linewidth,
+               q.P.empty() ? p.P : q.P,
+               q.font.empty() ? p.font : q.font,
+               q.fontsize == 0.0 ? p.fontsize : q.fontsize,
+               q.lineskip == 0.0 ? p.lineskip : q.lineskip,
+               colorspace,P.r,P.g,P.b,P.grey,
+               q.pattern == DEFPAT ? p.pattern : q.pattern,
+               q.fillrule == DEFFILL ? p.fillrule : q.fillrule,
+               q.baseline == DEFBASE ? p.baseline : q.baseline,
+               q.transparency == DEFTRANSP ? p.transparency : q.transparency,
+               q.linecap == DEFCAP ? p.linecap : q.linecap,
+               q.linejoin == DEFJOIN ? p.linejoin : q.linejoin,
+               q.miterlimit == DEFMITER ? p.miterlimit : q.miterlimit,
+               q.overwrite == DEFWRITE ? p.overwrite : q.overwrite,
+               q.t.isNull() ? p.t : q.t);
+  }
+
+  friend pen interpolate(const pen& p, const pen& q, double t) {
+    pen P=p;
+    pen Q=q;
+  
+    if(P.color == PATTERN && P.pattern.empty()) P.color=DEFCOLOR;
+    ColorSpace colorspace=(ColorSpace) max((Int) P.color,(Int) Q.color);
+  
+    switch(colorspace) {
+      case PATTERN:
+      case INVISIBLE:
+      case DEFCOLOR:
+      case GRAYSCALE:
+        break;
+      case RGB:
+      {
+        if(P.color == GRAYSCALE) P.greytorgb();
+        else if(Q.color == GRAYSCALE) Q.greytorgb();
+        break;
+      }
+      
+      case CMYK:
+      {
+        if(P.color == GRAYSCALE) P.greytocmyk();
+        else if(Q.color == GRAYSCALE) Q.greytocmyk();
+        
+        if(P.color == RGB) P.rgbtocmyk();
+        else if(Q.color == RGB) Q.rgbtocmyk();
+        break;
+      }
+    }
+    
+    return (1-t)*P+t*Q;
   }
 
   friend bool operator == (const pen& p, const pen& q) {
@@ -678,14 +764,15 @@ public:
       && p.colorspace() == q.colorspace()
       && (!(p.grayscale() || p.cmyk()) || p.gray() == q.gray())
       && (!(p.rgb() || p.cmyk()) || (p.red() == q.red() &&
-				     p.green() == q.green() &&
-				     p.blue() == q.blue()))
+                                     p.green() == q.green() &&
+                                     p.blue() == q.blue()))
       && p.pattern == q.pattern
       && p.Fillrule() == q.Fillrule()
       && p.Baseline() == q.Baseline()
       && p.transp() == q.transp()
       && p.cap() == q.cap()
       && p.join() == q.join()
+      && p.miter() == q.miter()
       && p.Overwrite() == q.Overwrite()
       && p.t == q.t;
   }
@@ -695,7 +782,11 @@ public:
   }
   
   friend ostream& operator << (ostream& out, const pen& p) {
-    out << "([" << p.line.pattern << "]";
+    out << "(";
+    if(p.line == DEFLINE)
+      out << p.line.pattern;
+    else
+      out << "[" << p.line.pattern << "]";
     if(p.line.offset)
       out << p.line.offset;
     if(!p.line.scale)
@@ -710,6 +801,8 @@ public:
       out << ", linecap=" << Cap[p.linecap];
     if(p.linejoin != DEFJOIN)
       out << ", linejoin=" << Join[p.linejoin];
+    if(p.miterlimit != DEFMITER)
+      out << ", miterlimit=" << p.miterlimit;
     if(!p.font.empty())
       out << ", font=\"" << p.font << "\"";
     if(p.fontsize)
@@ -722,10 +815,10 @@ public:
       out << ", gray=" << p.grey;
     else if(p.color == RGB)
       out << ", red=" << p.red() << ", green=" << p.green() 
-	  << ", blue=" << p.blue();
+          << ", blue=" << p.blue();
     else if(p.color == CMYK)
       out << ", cyan=" << p.cyan() << ", magenta=" << p.magenta() 
-	  << ", yellow=" << p.yellow() << ", black=" << p.black();
+          << ", yellow=" << p.yellow() << ", black=" << p.black();
     if(p.pattern != DEFPAT)
       out << ", pattern=" << "\"" << p.pattern << "\"";
     if(p.fillrule != DEFFILL)
@@ -765,7 +858,7 @@ public:
       shift = pair(0,0);
     } else {
       double xx = t.getxx(), xy = t.getxy(),
-             yx = t.getyx(), yy = t.getyy();
+        yx = t.getyx(), yy = t.getyy();
 
       // These are the maximum x and y values that a linear transform can map
       // a point in the unit circle.  This can be proven by the Lagrange
