@@ -20,6 +20,7 @@ class picture : public gc {
 private:
   bool labels;
   size_t lastnumber;
+  size_t lastnumber3;
   transform T; // Keep track of accumulative picture transform
   bbox b;
   bbox b_cached;   // Cached bounding box
@@ -31,10 +32,13 @@ private:
   static double paperWidth,paperHeight;
 
 public:
+  bbox3 b3; // 3D bounding box
+  
   typedef mem::list<drawElement*> nodelist;
   nodelist nodes;
   
-  picture() : labels(false), lastnumber(0), T(identity), transparency(false) {}
+  picture() : labels(false), lastnumber(0), lastnumber3(0), T(identity),
+              transparency(false) {}
   
   // Destroy all of the owned picture objects.
   ~picture();
@@ -53,10 +57,17 @@ public:
   void prepend(picture &pic);
   
   bool havelabels();
+  bool have3D();
+
   bbox bounds();
+  bbox3 bounds3();
 
-  void texinit();
-
+  // Projected bounds of 3d picture given transform3 t (not cached).
+  pair bounds(double (*m)(double, double),
+              double (*x)(const triple&, double*),
+              double (*y)(const triple&, double*),
+              double *t=NULL);
+  
   bool Transparency() {
     return transparency;
   }
@@ -64,18 +75,33 @@ public:
   int epstopdf(const string& epsname, const string& pdfname);
   
   bool texprocess(const string& texname, const string& tempname,
-		  const string& prefix, const pair& bboxshift); 
+                  const string& prefix, const pair& bboxshift); 
     
   bool postprocess(const string& prename, const string& outname, 
-		   const string& outputformat, double magnification,
-		   bool wait, bool view);
+                   const string& outputformat, double magnification,
+                   bool wait, bool view);
     
   // Ship the picture out to PostScript & TeX files.
   bool shipout(picture* preamble, const string& prefix,
-	       const string& format, double magnification=0.0,
-	       bool wait=false, bool view=true);
+               const string& format, double magnification=0.0,
+               bool wait=false, bool view=true);
  
+  void render(GLUnurbs *nurb, double size2,
+              const triple &Min, const triple& Max, double perspective,
+              bool transparent) const;
+  bool shipout3(const string& prefix, const string& format,
+                double width, double height, double angle, double zoom,
+                const triple& m, const triple& M, const pair& shift, double *t,
+                double *background, size_t nlights, triple *lights,
+                double *diffuse, double *ambient, double *specular,
+                bool viewportlighting, bool view);
+  
+  bool shipout3(const string& prefix); // Embedded PRC
+  
+  bool reloadPDF(const string& Viewer, const string& outname) const;
+  
   picture *transformed(const transform& t);
+  picture *transformed(const vm::array& t);
   
   bool null() {
     return nodes.empty();
@@ -88,6 +114,15 @@ inline picture *transformed(const transform& t, picture *p)
   return p->transformed(t);
 }
 
+inline picture *transformed(const vm::array& t, picture *p)
+{
+  return p->transformed(t);
+}
+
+void texinit();
+
+const char *texpathmessage();
+  
 } //namespace camp
 
 #endif
