@@ -1,18 +1,18 @@
 /* Pipestream: A simple C++ interface to UNIX pipes
-   Version 0.01
-   Copyright (C) 2005-2006 John C. Bowman
+   Version 0.04
+   Copyright (C) 2005-2009 John C. Bowman
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
+   it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU Lesser General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
+   You should have received a copy of the GNU Lesser General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */
 
@@ -26,6 +26,7 @@
 #include <sstream>
 #include <unistd.h>
 #include <signal.h>
+#include <fcntl.h>
 
 #include "common.h"
 #include "errormsg.h"
@@ -33,8 +34,6 @@
 #include "util.h"
 #include "interact.h"
 
-char **args(const char *command);
-  
 // bidirectional stream for reading and writing to pipes
 class iopipestream {
 protected:
@@ -43,7 +42,9 @@ protected:
   static const int BUFSIZE=SHRT_MAX;
   char buffer[BUFSIZE];
   int pid;
+  bool Running;
   bool pipeopen;
+  bool pipein;
   ostringstream transcript;
 public:
   
@@ -61,75 +62,85 @@ public:
   }
   
   void open(const char *command, const char *hint=NULL,
-	    const char *application="", int out_fileno=STDOUT_FILENO) {
+            const char *application="", int out_fileno=STDOUT_FILENO) {
     if(pipe(in) == -1) {
-      cerr << "in pipe failed: " << command << endl;
-      exit(-1);
+      ostringstream buf;
+      buf << "in pipe failed: " << command << endl;
+      camp::reportError(buf);
     }
 
     if(pipe(out) == -1) {
       ostringstream buf;
-      cerr << "out pipe failed: " << command << endl;
-      exit(-1);
+      buf << "out pipe failed: " << command << endl;
+      camp::reportError(buf);
     }
-    
     cout.flush(); // Flush stdout to avoid duplicate output.
     
-    int wrapperpid;
-    
-    // Portable way of forking that avoids zombie child processes
-    if((wrapperpid=fork()) < 0) {
-      cerr << "fork failed: " << command << endl;
-      exit(-1);
+    if((pid=fork()) < 0) {
+      ostringstream buf;
+      buf << "fork failed: " << command << endl;
+      camp::reportError(buf);
     }
     
-    if(wrapperpid == 0) {
-      if((pid=fork()) < 0) {
-	cerr << "fork failed: " << command << endl;
-	exit(-1);
-      }
-    
-      if(pid == 0) { 
-	if(interact::interactive) signal(SIGINT,SIG_IGN);
-	close(in[1]);
-	close(out[0]);
-	close(STDIN_FILENO);
-	close(out_fileno);
-	dup2(in[0],STDIN_FILENO);
-	dup2(out[1],out_fileno);
-	close(in[0]);
-	close(out[1]);
-	char **argv=args(command);
-	if(argv) execvp(argv[0],argv);
-	execError(command,hint,application);
-	kill(wrapperpid,SIGTERM);
-	exit(-1);
-      }
-      exit(0);
-    } else {
-      close(out[1]);
+    if(pid == 0) { 
+      if(interact::interactive) signal(SIGINT,SIG_IGN);
+      close(in[1]);
+      close(out[0]);
+      close(STDIN_FILENO);
+      close(out_fileno);
+      dup2(in[0],STDIN_FILENO);
+      dup2(out[1],out_fileno);
       close(in[0]);
-      *buffer=0;
-      pipeopen=true;
-      waitpid(wrapperpid,NULL,0);
+      close(out[1]);
+      char **argv=args(command);
+      if(argv) execvp(argv[0],argv);
+      execError(command,hint,application);
+      kill(0,SIGTERM);
+      _exit(-1);
     }
+    close(out[1]);
+    close(in[0]);
+    *buffer=0;
+    pipeopen=true;
+    pipein=true;
+    Running=true;
   }
 
+  void block(bool block=true) {
+    if(pipeopen) {
+      int flags=fcntl(out[0],F_GETFL);
+      if(block)
+        fcntl(out[0],F_SETFL,flags & ~O_NONBLOCK);
+      else
+        fcntl(out[0],F_SETFL,flags | O_NONBLOCK);
+    }
+  }
+  
   bool isopen() {return pipeopen;}
   
   iopipestream(): pid(0), pipeopen(false) {}
   
   iopipestream(const char *command, const char *hint=NULL,
-	       const char *application="", int out_fileno=STDOUT_FILENO) :
+               const char *application="", int out_fileno=STDOUT_FILENO) :
     pid(0), pipeopen(false) {
     open(command,hint,application,out_fileno);
   }
   
+  void eof() {
+    if(pipeopen && pipein) {
+      close(in[1]);
+      pipein=false;
+    }
+  }
+  
   virtual void pipeclose() {
     if(pipeopen) {
-      close(in[1]);
-//      close(out[0]);
+      kill(pid,SIGQUIT);
+      eof();
+      close(out[0]);
+      Running=false;
       pipeopen=false;
+      waitpid(pid,NULL,0); // Avoid zombies.
     }
   }
   
@@ -143,20 +154,27 @@ public:
     ssize_t size=BUFSIZE-1;
     for(;;) {
       if((nc=read(out[0],p,size)) < 0) {
-	camp::reportError("read from pipe failed");
+        if(errno == EAGAIN) {p[0]=0; break;}
+        else camp::reportError("read from pipe failed");
       }
       p[nc]=0;
-      if(nc == 0) break;
+      if(nc == 0) {
+        if(waitpid(pid, NULL, WNOHANG) == pid)
+          Running=false;
+        break;
+      }
       if(nc > 0) {
-	if(settings::verbose > 2) cerr << p;
-	if(strchr(p,'\n')) break;
-	p += nc;
-	size -= nc;
+        if(settings::verbose > 2) cerr << p;
+        if(strchr(p,'\n')) break;
+        p += nc;
+        size -= nc;
       }
     }
     return p+nc-buffer;
   }
 
+  bool running() {return Running;}
+  
   typedef iopipestream& (*imanip)(iopipestream&);
   iopipestream& operator << (imanip func) { return (*func)(*this); }
   
@@ -167,7 +185,7 @@ public:
   }
   
   bool tailequals(const char *buf, size_t len, const char *prompt,
-		  size_t plen) {
+                  size_t plen) {
     const char *a=buf+len;
     const char *b=prompt+plen;
     while(b >= prompt) {
@@ -189,8 +207,8 @@ public:
     while((p=strchr(p,'\n')) != NULL) {
       ++p;
       if(strncmp(p,abort,alen) == 0) {
-	clear();
-	return true;
+        clear();
+        return true;
       }
     }
     return false;
@@ -204,15 +222,15 @@ public:
     unsigned int n=0;
     if(abort) {
       for(;;) {
-	if(abort[n]) n++;
-	else break;
+        if(abort[n]) n++;
+        else break;
       }
     }
     
     do {
       len=readbuffer();
       for(unsigned int i=0; i < n; ++i) 
-	if(checkabort(abort[i])) return i+1;
+        if(checkabort(abort[i])) return i+1;
     } while (!tailequals(buffer,len,prompt,plen));
     return 0;
   }
@@ -221,19 +239,19 @@ public:
     for(;;) {
       int status;
       if (waitpid(pid, &status, 0) == -1) {
-	if (errno == ECHILD) return 0;
-	if (errno != EINTR) {
-	  ostringstream buf;
-	  buf << "Process " << pid << " failed";
-	  camp::reportError(buf);
-	}
+        if (errno == ECHILD) return 0;
+        if (errno != EINTR) {
+          ostringstream buf;
+          buf << "Process " << pid << " failed";
+          camp::reportError(buf);
+        }
       } else {
-	if(WIFEXITED(status)) return WEXITSTATUS(status);
-	else {
-	  ostringstream buf;
-	  buf << "Process " << pid << " exited abnormally";
-	  camp::reportError(buf);
-	}
+        if(WIFEXITED(status)) return WEXITSTATUS(status);
+        else {
+          ostringstream buf;
+          buf << "Process " << pid << " exited abnormally";
+          camp::reportError(buf);
+        }
       }
     }
   }
@@ -241,8 +259,10 @@ public:
   void Write(const string &s) {
     ssize_t size=s.length();
     if(settings::verbose > 2) cerr << s;
-    if(write(in[1],s.c_str(),size) != size)
-      camp::reportError("write to pipe failed");
+    if(write(in[1],s.c_str(),size) != size) {
+      camp::reportFatal("write to pipe failed");
+    }
+    
   }
   
   iopipestream& operator << (const string& s) {
