@@ -26,20 +26,20 @@
 #include "process.h"
 
 namespace camp {
-  pen& defaultpen() {
-    return processData().defaultpen;
-  }
+pen& defaultpen() {
+  return processData().defaultpen;
+}
 }
 
 namespace run {
-  void cleanup();
-  void exitFunction(vm::stack *Stack);
-  void updateFunction(vm::stack *Stack);
-  void purge();
+void cleanup();
+void exitFunction(vm::stack *Stack);
+void updateFunction(vm::stack *Stack);
+void purge(Int divisor=0);
 }
 
 namespace vm {
-  bool indebugger;  
+bool indebugger;  
 }
 
 using namespace settings;
@@ -67,9 +67,10 @@ processDataStruct &processData() {
 void init(bool resetpath=true)
 {
   vm::indebugger=false;
+  uptodate=false;
   if(resetpath)
     setPath("");  /* On second and subsequent calls, sets the path
-		     to what it was when the program started. */
+                     to what it was when the program started. */
 }
 
 // This helper class does nothing but call the interactiveTrans method of the
@@ -109,8 +110,8 @@ bool runRunnable(runnable *r, coenv &e, istack &s, transMode tm=TRANS_NORMAL) {
   e.e.beginScope();
 
   lambda *codelet= tm==TRANS_INTERACTIVE ?
-                       interactiveRunnable(r).transAsCodelet(e) :
-                       r->transAsCodelet(e);
+    interactiveRunnable(r).transAsCodelet(e) :
+    r->transAsCodelet(e);
   em.sync();
   if(!em.errors()) {
     if(getSetting<bool>("translate")) print(cout,codelet->code);
@@ -185,8 +186,7 @@ public:
       postRun(e,s);
 
     } catch(std::bad_alloc&) {
-      cerr << "error: out of memory" << endl;
-      em.statusError();
+      outOfMemory();
     } catch(quit) {
     } catch(handled_error) {
       em.statusError();
@@ -200,14 +200,15 @@ public:
   virtual void process(bool purge=false) {
     if (!interactive && getSetting<bool>("parseonly"))
       doParse();
-    else if (getSetting<bool>("listvariables"))
-      doList();
     else {
       // This is not done in preRun as it is not an optional step.
       processDataStruct data;
       processDataStack.push(&data);
-
-      doRun(purge);
+      
+      if (getSetting<bool>("listvariables"))
+        doList();
+      else
+        doRun(purge);
 
       processDataStack.pop();
     }
@@ -315,8 +316,8 @@ public:
   ifile(const string& filename)
     : itree(filename),
       filename(filename),
-      outname((string) (filename == "-" ? "out" :
-			stripDir(stripExt(string(filename), suffix)))) {}
+      outname((string) (filename == "-" ? settings::outname() :
+                        stripDir(stripExt(string(filename), suffix)))) {}
   
   block *buildTree() {
     return !filename.empty() ? parser::parseFile(filename,"Loading") : 0;
@@ -355,8 +356,9 @@ public:
 };
 
 void printGreeting() {
-    cout << "Welcome to " << PROGRAM << " version " << VERSION
-	 << " (to view the manual, type help)" << endl;
+  if(!getSetting<bool>("quiet"))
+    cout << "Welcome to " << PROGRAM << " version " << VERSION << SVN_REVISION
+         << " (to view the manual, type help)" << endl;
 }
 
 
@@ -427,8 +429,8 @@ size_type stringPos(const string line, size_type start) {
   if (min(startComment,startLineComment) < pos)
     return stringPos(line,
                      startComment < startLineComment ?
-                         endOfComment(line, startComment+2) :
-                         npos);
+                     endOfComment(line, startComment+2) :
+                     npos);
   else
     // Nothing to skip over - the symbol actually starts a string.
     return pos;
@@ -466,8 +468,8 @@ string endString(const string line, size_type start) {
         case '\"':
           return line+"\" \'\\n\' \"";
         default:
-           assert(False);
-           return line;
+          assert(False);
+          return line;
       }
     }
     else {
@@ -492,8 +494,8 @@ size_type slashPos(const string line, size_type start) {
   if (min(startComment,startString) < pos)
     return slashPos(line,
                     startComment < startString ?
-                        endOfComment(line, startComment+2) :
-                        endOfString(line[startString], line, startString+1));
+                    endOfComment(line, startComment+2) :
+                    endOfString(line[startString], line, startString+1));
   else
     // Nothing to skip over - the // actually starts a comment.
     return pos;
@@ -629,7 +631,6 @@ class iprompt : public icore {
       restart=true;
       startline="";
 
-      uptodate=true;
       run::purge();
 
       return true;
@@ -647,7 +648,7 @@ class iprompt : public icore {
 
 
   bool input(commandLine cl) {
-    string prefix="erase(); include ";
+    string prefix="include ";
     string line=prefix+cl.rest;
     running=false;
     restart=true;
@@ -656,7 +657,7 @@ class iprompt : public icore {
   }
 
   void initCommands() {
-#define ADDCOMMAND(name, func) \
+#define ADDCOMMAND(name, func)                  \
     commands[#name]=&iprompt::func
 
     // keywords.pl looks for ADDCOMMAND to identify special commands in the
@@ -665,8 +666,8 @@ class iprompt : public icore {
     ADDCOMMAND(q,q);
     ADDCOMMAND(exit,exit);
     ADDCOMMAND(reset,reset);
-    ADDCOMMAND(help, help);
-    ADDCOMMAND(input, input);
+    ADDCOMMAND(help,help);
+    ADDCOMMAND(input,input);
 
 #undef ADDCOMMAND
   }
@@ -719,7 +720,7 @@ class iprompt : public icore {
 
     // If the line ends in a slash, get more input.
     return isSlashed(line) ? line+"\n"+getline(true) :
-                             line;
+      line;
   }
 
   // Continue taking input for a line until it properly parses, or a syntax
@@ -756,8 +757,8 @@ class iprompt : public icore {
         i.run(e,s,TRANS_INTERACTIVE);
       }
 
-      if(!uptodate)
-        run::updateFunction(&s);
+      run::updateFunction(&s);
+      uptodate=false;
 
     } catch(handled_error) {
       vm::indebugger=false;
@@ -816,14 +817,14 @@ public:
 
     do {
       try {
-	init(false);
-	restart=false;
-	icore::process();
+        init(false);
+        restart=false;
+        icore::process();
       } catch(interrupted&) {
-	em.Interrupt(false);
-	restart=true;
+        em.Interrupt(false);
+        restart=true;
       } catch(eof&) {
-	restart=false;
+        restart=false;
       }
     } while(restart);
       
@@ -865,6 +866,9 @@ void runPromptEmbedded(trans::coenv &e, istack &s) {
 }
 
 void doUnrestrictedList() {
+  processDataStruct data;
+  processDataStack.push(&data);
+  
   genv ge;
   env base_env(ge);
   coder base_coder;
@@ -873,6 +877,7 @@ void doUnrestrictedList() {
   if (getSetting<bool>("autoplain"))
     absyntax::autoplainRunnable()->trans(e);
 
+  processDataStack.pop();
   e.e.list(0);
 }
 
