@@ -9,6 +9,7 @@
 #include <ctime>
 #include <iomanip>
 #include <sstream>
+#include <zlib.h>
 
 #include "psfile.h"
 #include "settings.h"
@@ -25,39 +26,100 @@ namespace camp {
 void checkColorSpace(ColorSpace colorspace)
 {
   switch(colorspace) {
-  case DEFCOLOR:
-  case INVISIBLE:
-    reportError("Cannot shade with invisible pen");
-  case PATTERN:
-    reportError("Cannot shade with pattern");
-    break;
-  default:
-    break;
+    case DEFCOLOR:
+    case INVISIBLE:
+      reportError("Cannot shade with invisible pen");
+    case PATTERN:
+      reportError("Cannot shade with pattern");
+      break;
+    default:
+      break;
   }
 }
     
-  psfile::psfile(const string& filename, bool pdfformat)
-    : filename(filename), pdfformat(pdfformat), pdf(false),
-      transparency(false), out(NULL)
+psfile::psfile(const string& filename, bool pdfformat)
+  : filename(filename), pdfformat(pdfformat), pdf(false),
+    transparency(false), buffer(NULL), out(NULL) 
 {
   if(filename.empty()) out=&cout;
   else out=new ofstream(filename.c_str());
-  if(!out || !*out) {
-    cerr << "Cannot write to " << filename << endl;
-    throw handled_error();
+  if(!out || !*out)
+    reportError("Cannot write to "+filename);
+}
+
+static const char *inconsistent="inconsistent colorspaces";
+  
+void psfile::writefromRGB(unsigned char r, unsigned char g, unsigned char b, 
+                          ColorSpace colorspace, size_t ncomponents) 
+{
+  static const double factor=1.0/255.0;
+  pen p(r*factor,g*factor,b*factor);
+  p.convert();
+  if(!p.promote(colorspace))
+    reportError(inconsistent);
+  write(&p,ncomponents);
+}
+
+inline unsigned char average(unsigned char *a, size_t dx, size_t dy)
+{
+  return ((unsigned) a[0]+(unsigned) a[dx]+(unsigned) a[dy]+
+          (unsigned) a[dx+dy])/4;
+}
+
+void psfile::dealias(unsigned char *a, size_t width, size_t height, size_t n,
+                     bool convertrgb, ColorSpace colorspace)
+{
+  // Dealias all but the last row and column of pixels.
+  size_t istop=width-1;
+  size_t jstop=height-1;
+  if(convertrgb) {
+    size_t nwidth=3*width;
+    for(size_t j=0; j < height; ++j) {
+      unsigned char *aj=a+nwidth*j;
+      for(size_t i=0; i < width; ++i) {
+        unsigned char *ai=aj+3*i;
+        if(i < istop && j < jstop)
+          writefromRGB(average(ai,3,nwidth),
+                       average(ai+1,3,nwidth),
+                       average(ai+2,3,nwidth),colorspace,n);
+        else
+          writefromRGB(ai[0],ai[1],ai[2],colorspace,n);
+      }
+    }
+  } else {
+    size_t nwidth=n*width;
+    for(size_t j=0; j < jstop; ++j) {
+      unsigned char *aj=a+nwidth*j;
+      for(size_t i=0; i < istop; ++i) {
+        unsigned char *ai=aj+n*i;
+        for(size_t k=0; k < n; ++k)
+          ai[k]=average(ai+k,n,nwidth);
+      }
+    }
   }
 }
 
+void psfile::writeCompressed(const unsigned char *a, size_t size)
+{  
+  uLongf compressedSize=compressBound(size);
+  Bytef *compressed=new Bytef[compressedSize];
+  
+  if(compress(compressed,&compressedSize,a,size) != Z_OK)
+    reportError("image compression failed");
+
+  encode85 e(out);
+  for(size_t i=0; i < compressedSize; ++i)
+    e.put(compressed[i]);
+}
+  
 void psfile::close()
 {
   if(out) {
     out->flush();
     if(!filename.empty()) {
-      if(!out->good()) {
-	ostringstream msg;
-	msg << "Cannot write to " << filename;
-	reportError(msg);
-      }
+      if(!out->good())
+        // Don't call reportError since this may be called on handled_error.
+        reportFatal("Cannot write to "+filename);
       delete out;
       out=NULL;
     }
@@ -80,7 +142,7 @@ void psfile::prologue(const bbox& box)
   header();
   BoundingBox(box);
   *out << "%%Creator: " << settings::PROGRAM << " " << settings::VERSION
-       <<  newl;
+       << SVN_REVISION <<  newl;
 
   time_t t; time(&t);
   struct tm *tt = localtime(&t);
@@ -95,7 +157,10 @@ void psfile::prologue(const bbox& box)
   *out << "%%Page: 1 1" << newl;
   
   if(!pdfformat)
-    *out << "/Setlinewidth {0 exch dtransform dup abs 1 lt {pop 0}{round} ifelse idtransform setlinewidth pop} bind def" << newl;
+    *out 
+      << "/Setlinewidth {0 exch dtransform dup abs 1 lt {pop 0}{round} ifelse"
+      << newl
+      << "idtransform setlinewidth pop} bind def" << newl;
 }
 
 void psfile::epilogue()
@@ -105,31 +170,29 @@ void psfile::epilogue()
 }
 
 void psfile::setcolor(const pen& p, const string& begin="",
-		      const string& end="")
+                      const string& end="")
 {
   if(p.cmyk() && (!lastpen.cmyk() ||
-		  (p.cyan() != lastpen.cyan() || 
-		   p.magenta() != lastpen.magenta() || 
-		   p.yellow() != lastpen.yellow() ||
-		   p.black() != lastpen.black()))) {
+                  (p.cyan() != lastpen.cyan() || 
+                   p.magenta() != lastpen.magenta() || 
+                   p.yellow() != lastpen.yellow() ||
+                   p.black() != lastpen.black()))) {
     *out << begin << p.cyan() << " " << p.magenta() << " " << p.yellow() << " " 
-	 << p.black() << (pdf ? " k" : " setcmykcolor") << end << newl;
+         << p.black() << (pdf ? " k" : " setcmykcolor") << end << newl;
   } else if(p.rgb() && (!lastpen.rgb() || 
-			(p.red() != lastpen.red() || 
-			 p.green() != lastpen.green() || 
-			 p.blue() != lastpen.blue()))) {
+                        (p.red() != lastpen.red() || 
+                         p.green() != lastpen.green() || 
+                         p.blue() != lastpen.blue()))) {
     *out << begin << p.red() << " " << p.green() << " " << p.blue()
-	 << (pdf ? " rg" : " setrgbcolor") << end << newl;
+         << (pdf ? " rg" : " setrgbcolor") << end << newl;
   } else if(p.grayscale() && (!lastpen.grayscale() ||
-			      p.gray() != lastpen.gray())) {
+                              p.gray() != lastpen.gray())) {
     *out << begin << p.gray() << (pdf ? " g" : " setgray") << end << newl;
   }
 }
   
-void psfile::setpen(pen p)
+void psfile::setopacity(const pen& p)
 {
-  p.convert();
-    
   if(p.blend() != lastpen.blend()) {
     *out << "/" << p.blend() << " .setblendmode" << newl;
     transparency=true;
@@ -140,6 +203,16 @@ void psfile::setpen(pen p)
     transparency=true;
   }
   
+  lastpen.settransparency(p);
+}
+
+  
+void psfile::setpen(pen p)
+{
+  p.convert();
+    
+  setopacity(p);
+  
   if(!p.fillpattern().empty() && p.fillpattern() != lastpen.fillpattern()) 
     *out << p.fillpattern() << " setpattern" << newl;
   else setcolor(p);
@@ -147,7 +220,7 @@ void psfile::setpen(pen p)
   // Defer dynamic linewidth until stroke time in case currentmatrix changes.
   if(p.width() != lastpen.width()) {
     *out << p.width() << (pdfformat ? " setlinewidth" : " Setlinewidth") 
-	 << newl;
+         << newl;
   }
     
   if(p.cap() != lastpen.cap()) {
@@ -158,22 +231,26 @@ void psfile::setpen(pen p)
     *out << p.join() << " setlinejoin" << newl;
   }
     
+  if(p.miter() != lastpen.miter()) {
+    *out << p.miter() << " setmiterlimit" << newl;
+  }
+    
   if(p.stroke() != lastpen.stroke() || 
      p.linetype().offset != lastpen.linetype().offset) {
     out->setf(std::ios::fixed);
     *out << "[" << p.stroke() << "] " << std::setprecision(6) 
-	 << p.linetype().offset << " setdash" << newl;
+         << p.linetype().offset << " setdash" << newl;
     out->unsetf(std::ios::fixed);
   }
     
   lastpen=p;
 }
 
-void psfile::write(pen p)
+void psfile::write(const pen& p)
 {
   if(p.cmyk())
     *out << p.cyan() << " " << p.magenta() << " " << p.yellow() << " " 
-	 << p.black();
+         << p.black();
   else if(p.rgb())
     *out << p.red() << " " << p.green() << " " << p.blue();
   else if(p.grayscale())
@@ -187,28 +264,25 @@ void psfile::write(path p, bool newPath)
 
   if(newPath) newpath();
 
-  if (n == 1) {
-    moveto(p.point((Int) 0));
-    lineto(p.point((Int) 0));
-    return;
-  }
+  pair z0=p.point((Int) 0);
 
   // Draw points
-  moveto(p.point((Int) 0));
-  for (Int i = 1; i < n; i++) {
+  moveto(z0);
+  
+  for(Int i = 1; i < n; i++) {
     if(p.straight(i-1)) lineto(p.point(i));
-    else curveto(p.postcontrol(i-1), p.precontrol(i), p.point(i));
+    else curveto(p.postcontrol(i-1),p.precontrol(i),p.point(i));
   }
 
-  if (p.cyclic()) {
-    if(p.straight(n-1)) lineto(p.point((Int) 0));
-    else curveto(p.postcontrol(n-1), p.precontrol((Int) 0), p.point((Int) 0));
+  if(p.cyclic()) {
+    if(p.straight(n-1)) lineto(z0);
+    else curveto(p.postcontrol(n-1),p.precontrol((Int) 0),z0);
     closepath();
-  }    
+  } else {
+    if(n == 1) lineto(z0);
+  }
 }
 
-static const char *inconsistent="inconsistent colorspaces";
-  
 void psfile::latticeshade(const vm::array& a, const bbox& b)
 {
   checkLevel();
@@ -217,12 +291,12 @@ void psfile::latticeshade(const vm::array& a, const bbox& b)
   
   array *a0=read<array *>(a,0);
   size_t m=a0->size();
-  setfirstpen(*a0);
+  setfirstopacity(*a0);
   
   ColorSpace colorspace=maxcolorspace2(a);
   checkColorSpace(colorspace);
   
-  unsigned ncomponents=ColorComponents[colorspace];
+  size_t ncomponents=ColorComponents[colorspace];
   
   *out << "<< /ShadingType 1" << newl
        << "/Matrix ";
@@ -234,43 +308,46 @@ void psfile::latticeshade(const vm::array& a, const bbox& b)
        << "<< /FunctionType 0" << newl
        << "/Order 1" << newl
        << "/Domain [0 1 0 1]" << newl
-       << "/Range [" << newl;
-  for(unsigned i=0; i < ncomponents; ++i)
+       << "/Range [";
+  for(size_t i=0; i < ncomponents; ++i)
     *out << "0 1 ";
   *out << "]" << newl
        << "/Decode [";
-  for(unsigned i=0; i < ncomponents; ++i)
+  for(size_t i=0; i < ncomponents; ++i)
     *out << "0 1 ";
   *out << "]" << newl;
   *out << "/BitsPerSample 8" << newl;
   *out << "/Size [" << m << " " << n << "]" << newl
        << "/DataSource <" << newl;
+
+  beginHex();
   for(size_t i=n; i > 0;) {
     array *ai=read<array *>(a,--i);
     checkArray(ai);
     size_t aisize=ai->size();
     if(aisize != m) reportError("shading matrix must be rectangular");
     for(size_t j=0; j < m; j++) {
-	pen *p=read<pen *>(ai,j);
-	p->convert();
-	if(!p->promote(colorspace))
-	  reportError(inconsistent);
-	writeHex(p,ncomponents);
-      }
+      pen *p=read<pen *>(ai,j);
+      p->convert();
+      if(!p->promote(colorspace))
+        reportError(inconsistent);
+      writeHex(p,ncomponents);
     }
-  *out << ">" << newl
-       << ">>" << newl
+  }
+  endHex();
+
+  *out << ">>" << newl
        << ">>" << newl
        << "shfill" << newl;
 }
 
 // Axial and radial shading
 void psfile::gradientshade(bool axial, const ColorSpace &colorspace,
-			   const pen& pena, const pair& a, double ra,
-			   const pen& penb, const pair& b, double rb)
+                           const pen& pena, const pair& a, double ra,
+                           const pen& penb, const pair& b, double rb)
 {
   checkLevel();
-  setpen(pena);
+  setopacity(pena);
   checkColorSpace(colorspace);
   
   *out << "<< /ShadingType " << (axial ? "2" : "3") << newl
@@ -298,13 +375,13 @@ void psfile::gradientshade(bool axial, const ColorSpace &colorspace,
 }
   
 void psfile::gouraudshade(const array& pens, const array& vertices,
-			  const array& edges)
+                          const array& edges)
 {
   checkLevel();
   size_t size=pens.size();
   if(size == 0) return;
   
-  setfirstpen(pens);
+  setfirstopacity(pens);
   ColorSpace colorspace=maxcolorspace(pens);
 
   *out << "<< /ShadingType 4" << newl
@@ -328,7 +405,7 @@ void psfile::gouraudshade(const array& pens, const array& vertices,
  
 // Tensor-product patch shading
 void psfile::tensorshade(const array& pens, const array& boundaries,
-			 const array& z)
+                         const array& z)
 {
   checkLevel();
   size_t size=pens.size();
@@ -338,7 +415,7 @@ void psfile::tensorshade(const array& pens, const array& boundaries,
   array *p0=read<array *>(pens,0);
   if(checkArray(p0) != 4)
     reportError("4 pens required");
-  setfirstpen(*p0);
+  setfirstopacity(*p0);
   
   ColorSpace colorspace=maxcolorspace2(pens);
   checkColorSpace(colorspace);
@@ -363,17 +440,17 @@ void psfile::tensorshade(const array& pens, const array& boundaries,
     if(nz == 0) { // Coons patch
       static double nineth=1.0/9.0;
       for(Int j=0; j < 4; ++j) {
-	write(nineth*(-4.0*g.point(j)+6.0*(g.precontrol(j)+g.postcontrol(j))
-		      -2.0*(g.point(j-1)+g.point(j+1))
-		      +3.0*(g.precontrol(j-1)+g.postcontrol(j+1))
-		      -g.point(j+2)));
+        write(nineth*(-4.0*g.point(j)+6.0*(g.precontrol(j)+g.postcontrol(j))
+                      -2.0*(g.point(j-1)+g.point(j+1))
+                      +3.0*(g.precontrol(j-1)+g.postcontrol(j+1))
+                      -g.point(j+2)));
       }
     } else {
       array *zi=read<array *>(z,i);
       if(checkArray(zi) != 4)
-	reportError("specify 4 internal control points for each path");
+        reportError("specify 4 internal control points for each path");
       for(Int j=0; j < 4; ++j)
-	write(read<pair>(zi,j));
+        write(read<pair>(zi,j));
     }
     
     array *pi=read<array *>(pens,i);
@@ -383,7 +460,7 @@ void psfile::tensorshade(const array& pens, const array& boundaries,
       pen *p=read<pen *>(pi,j);
       p->convert();
       if(!p->promote(colorspace))
-	reportError(inconsistent);
+        reportError(inconsistent);
       *out << " ";
       write(*p);
     }
@@ -395,7 +472,7 @@ void psfile::tensorshade(const array& pens, const array& boundaries,
        << "shfill" << newl;
 }
  
-inline unsigned byte(double r) // Map [0,1] to [0,255]
+inline unsigned char byte(double r) // Map [0,1] to [0,255]
 {
   if(r < 0.0) r=0.0;
   else if(r > 1.0) r=1.0;
@@ -404,35 +481,65 @@ inline unsigned byte(double r) // Map [0,1] to [0,255]
   return a;
 }
 
-void psfile::writeHex(pen *p, Int ncomponents) 
+void psfile::write(pen *p, size_t ncomponents) 
 {
   switch(ncomponents) {
-  case 0:
-    break;
-  case 1: 
-    writeHex(byte(p->gray())); 
-    *out << newl;
-    break;
-  case 3:
-    writeHex(byte(p->red())); 
-    writeHex(byte(p->green())); 
-    writeHex(byte(p->blue())); 
-    *out << newl;
-    break;
-  case 4:
-    writeHex(byte(p->cyan())); 
-    writeHex(byte(p->magenta())); 
-    writeHex(byte(p->yellow())); 
-    writeHex(byte(p->black())); 
-    *out << newl;
-  default:
-    break;
+    case 0:
+      break;
+    case 1: 
+      writeByte(byte(p->gray())); 
+      break;
+    case 3:
+      writeByte(byte(p->red())); 
+      writeByte(byte(p->green())); 
+      writeByte(byte(p->blue())); 
+      break;
+    case 4:
+      writeByte(byte(p->cyan())); 
+      writeByte(byte(p->magenta())); 
+      writeByte(byte(p->yellow())); 
+      writeByte(byte(p->black())); 
+    default:
+      break;
   }
 }
 
-void psfile::imageheader(double width, double height, ColorSpace colorspace)
+void psfile::writeHex(pen *p, size_t ncomponents) 
 {
-  unsigned ncomponents=ColorComponents[colorspace];
+  switch(ncomponents) {
+    case 0:
+      break;
+    case 1: 
+      write2(byte(p->gray())); 
+      writenewl();
+      break;
+    case 3:
+      write2(byte(p->red())); 
+      write2(byte(p->green())); 
+      write2(byte(p->blue())); 
+      writenewl();
+      break;
+    case 4:
+      write2(byte(p->cyan())); 
+      write2(byte(p->magenta())); 
+      write2(byte(p->yellow())); 
+      write2(byte(p->black())); 
+      writenewl();
+    default:
+      break;
+  }
+}
+
+string filter() 
+{
+  return settings::getSetting<Int>("level") >= 3 ? 
+    "1 (~>) /SubFileDecode filter /ASCII85Decode filter\n/FlateDecode" :
+    "1 (~>) /SubFileDecode filter /ASCII85Decode";
+}
+
+void psfile::imageheader(size_t width, size_t height, ColorSpace colorspace)
+{
+  size_t ncomponents=ColorComponents[colorspace];
   *out << "/Device" << ColorDeviceSuffix[colorspace] << " setcolorspace" 
        << newl
        << "<<" << newl
@@ -442,17 +549,17 @@ void psfile::imageheader(double width, double height, ColorSpace colorspace)
        << "/BitsPerComponent 8" << newl
        << "/Decode [";
   
-  for(unsigned i=0; i < ncomponents; ++i)
+  for(size_t i=0; i < ncomponents; ++i)
     *out << "0 1 ";
   
   *out << "]" << newl
        << "/ImageMatrix [" << width << " 0 0 " << height << " 0 0]" << newl
-       << "/DataSource currentfile /ASCIIHexDecode filter" << newl
+       << "/DataSource currentfile " << filter() << " filter" << newl
        << ">>" << newl
        << "image" << newl;
 }
 
-void psfile::image(const array& a, const array& P)
+void psfile::image(const array& a, const array& P, bool antialias)
 {
   size_t asize=a.size();
   size_t Psize=P.size();
@@ -462,12 +569,12 @@ void psfile::image(const array& a, const array& P)
   size_t a0size=a0->size();
   if(a0size == 0) return;
   
-  setfirstpen(P);
+  setfirstopacity(P);
   
   ColorSpace colorspace=maxcolorspace(P);
   checkColorSpace(colorspace);
   
-  unsigned ncomponents=ColorComponents[colorspace];
+  size_t ncomponents=ColorComponents[colorspace];
   
   imageheader(a0size,asize,colorspace);
     
@@ -476,14 +583,15 @@ void psfile::image(const array& a, const array& P)
   for(size_t i=0; i < asize; i++) {
     array *ai=read<array *>(a,i);
     for(size_t j=0; j < a0size; j++) {
-	double val=read<double>(ai,j);
-	if(val > max) max=val;
-	else if(val < min) min=val;
+      double val=read<double>(ai,j);
+      if(val > max) max=val;
+      else if(val < min) min=val;
     }
   }
   
   double step=(max == min) ? 0.0 : (Psize-1)/(max-min);
   
+  beginImage(ncomponents*a0size*asize);
   for(size_t i=0; i < asize; i++) {
     array *ai=read<array *>(a,i);
     for(size_t j=0; j < a0size; j++) {
@@ -492,15 +600,14 @@ void psfile::image(const array& a, const array& P)
       pen *p=read<pen *>(P,index < Psize ? index : Psize-1);
       p->convert();
       if(!p->promote(colorspace))
-	reportError(inconsistent);
-      writeHex(p,ncomponents);
+        reportError(inconsistent);
+      write(p,ncomponents);
     }
   }
-  
-  *out << ">" << endl;
+  endImage(antialias,a0size,asize,ncomponents);
 }
 
-void psfile::image(const array& a)
+void psfile::image(const array& a, bool antialias)
 {
   size_t asize=a.size();
   if(asize == 0) return;
@@ -509,27 +616,74 @@ void psfile::image(const array& a)
   size_t a0size=a0->size();
   if(a0size == 0) return;
   
-  setfirstpen(*a0);
+  setfirstopacity(*a0);
   
   ColorSpace colorspace=maxcolorspace2(a);
   checkColorSpace(colorspace);
   
-  unsigned ncomponents=ColorComponents[colorspace];
+  size_t ncomponents=ColorComponents[colorspace];
   
   imageheader(a0size,asize,colorspace);
     
+  beginImage(ncomponents*a0size*asize);
   for(size_t i=0; i < asize; i++) {
     array *ai=read<array *>(a,i);
     for(size_t j=0; j < a0size; j++) {
       pen *p=read<pen *>(ai,j);
       p->convert();
       if(!p->promote(colorspace))
-	reportError(inconsistent);
-      writeHex(p,ncomponents);
+        reportError(inconsistent);
+      write(p,ncomponents);
     }
   }
-  
-  *out << ">" << endl;
+  endImage(antialias,a0size,asize,ncomponents);
 }
   
+void psfile::outImage(bool antialias, size_t width, size_t height,
+                      size_t ncomponents)
+{
+  if(antialias) dealias(buffer,width,height,ncomponents);
+  if(settings::getSetting<Int>("level") >= 3)
+    writeCompressed(buffer,count);
+  else {
+    encode85 e(out);
+    for(size_t i=0; i < count; ++i)
+      e.put(buffer[i]);
+  }
+}
+  
+void psfile::rawimage(unsigned char *a, size_t width, size_t height,
+                      bool antialias)
+{
+  pen p(0.0,0.0,0.0);
+  p.convert();
+  ColorSpace colorspace=p.colorspace();
+  checkColorSpace(colorspace);
+  
+  size_t ncomponents=ColorComponents[colorspace];
+  
+  imageheader(width,height,colorspace);
+  
+  count=ncomponents*width*height;
+  if(colorspace == RGB) {
+    buffer=a;
+    outImage(antialias,width,height,ncomponents);
+  } else {
+    beginImage(count);
+    if(antialias)
+      dealias(a,width,height,ncomponents,true,colorspace);
+    else {
+      size_t height3=3*height;
+      for(size_t i=0; i < width; ++i) {
+        unsigned char *ai=a+height3*i;
+        for(size_t j=0; j < height; ++j) {
+          unsigned char *aij=ai+3*j;
+          writefromRGB(aij[0],aij[1],aij[2],colorspace,ncomponents);
+        }
+      }
+    }
+    endImage(false,width,height,ncomponents);
+  }
+}
+
 } //namespace camp
