@@ -24,28 +24,124 @@ namespace camp {
 inline void BoundingBox(std::ostream& s, const bbox& box) 
 {
   s << "%%BoundingBox: " << std::setprecision(0) << std::fixed 
-       << box.LowRes() << newl;
+    << box.LowRes() << newl;
   s.unsetf(std::ios::fixed);
   s << "%%HiResBoundingBox: " << std::setprecision(9) << box << newl;
 }
 
+// An ASCII85Encode filter.
+class encode85 {
+  ostream *out;
+  int tuple;
+  int pos;
+  int count;
+  static const int width=72;
+public:
+  encode85(ostream *out) : out(out), tuple(0), pos(0), count(0) {}
+  
+  ~encode85() {
+    if(count > 0)
+      encode(tuple, count);
+    if(pos+2 > width)
+      *out << '\n';
+    *out << "~>\n";
+  }
+private:  
+  void encode(unsigned int tuple, int count) {
+    unsigned char buf[5], *s=buf;
+    int i=5;
+    do {
+      *s++=tuple % 85;
+      tuple /= 85;
+    } while(--i > 0);
+    i=count;
+    do {
+      *out << (unsigned char) (*--s + '!');
+      if(pos++ >= width) {
+        pos=0;
+        *out << '\n';
+      }
+    } while(i-- > 0);
+  }
+
+public:  
+  void put(unsigned char c) {
+    switch (count++) {
+      case 0:
+        tuple |= (c << 24);
+        break;
+      case 1:
+        tuple |= (c << 16);
+        break;
+      case 2:
+        tuple |= (c <<  8);
+        break;
+      case 3:
+        tuple |= c;
+        if(tuple == 0) {
+          *out << 'z';
+          if(pos++ >= width) {
+            pos=0;
+            *out << '\n';
+          }
+        } else
+          encode(tuple, count);
+        tuple=0;
+        count=0;
+        break;
+    }
+  }
+};
+
 class psfile {
   string filename;
   bool pdfformat;    // Is final output format PDF?
   bool pdf;          // Output direct PDF?
   bool transparency; // Is transparency used?
+  unsigned char *buffer;
+  size_t count;
   mem::stack<pen> pens;
-
-  void write(transform t) {
-    if(!pdf) *out << "[";
-    *out << " " << t.getxx() << " " << t.getyx()
-	 << " " << t.getxy() << " " << t.getyy()
-	 << " " << t.getx() << " " << t.gety();
-    if(!pdf) *out << "]";
+  
+  void writeHex(pen *p, size_t ncomponents);
+  void write(pen *p, size_t ncomponents);
+  void writefromRGB(unsigned char r, unsigned char g, unsigned char b, 
+                    ColorSpace colorspace, size_t ncomponents);
+  
+  void writeCompressed(const unsigned char *a, size_t size);
+  void dealias(unsigned char *a, size_t width, size_t height, size_t n,
+               bool convertrgb=false, ColorSpace colorspace=DEFCOLOR);
+  
+  void beginHex() {
+    out->setf(std::ios::hex,std::ios::basefield);
+    out->fill('0');
   }
-
-  void writeHex(unsigned n) {
-    *out << std::hex << std::setw(2) << std::setfill('0') << n << std::dec;
+  
+  void endHex() {
+    out->setf(std::ios::dec,std::ios::basefield);
+    out->fill();
+    *out << ">" << endl;
+  }
+  
+  void beginImage(size_t n) {
+    buffer=new unsigned char[n];
+    count=0;
+  }
+  
+  void outImage(bool antialias, size_t width, size_t height,
+                size_t ncomponents);
+  
+  void endImage(bool antialias, size_t width, size_t height,
+                size_t ncomponents) {
+    outImage(antialias,width,height,ncomponents);
+    delete[] buffer;
+  }
+  
+  void writeByte(unsigned char n) {
+    buffer[count++]=n;
+  }
+  
+  void write2(unsigned n) {
+    *out << std::setw(2) << n;
   }
   
 protected:
@@ -54,6 +150,7 @@ protected:
   
 public: 
   psfile(const string& filename, bool pdfformat);
+  
   psfile() {pdf=settings::pdf(settings::getSetting<string>("tex"));}
 
   ~psfile();
@@ -72,6 +169,10 @@ public:
     *out << " " << x;
   }
 
+  void writenewl() {
+    *out << newl;
+  }
+  
   bool Transparency() {
     return transparency;
   }
@@ -80,18 +181,25 @@ public:
     *out << " " << z.getx() << " " << z.gety();
   }
 
-  void writeHex(pen *p, Int ncomponents);
-  
+  void write(transform t) {
+    if(!pdf) *out << "[";
+    *out << " " << t.getxx() << " " << t.getyx()
+         << " " << t.getxy() << " " << t.getyy()
+         << " " << t.getx() << " " << t.gety();
+    if(!pdf) *out << "]";
+  }
+
   void resetpen() {
     lastpen=pen(initialpen);
     lastpen.convert();
   }
   
   void setcolor(const pen& p, const string& begin, const string& end);
+  void setopacity(const pen& p);
 
   void setpen(pen p);
   
-  void write(pen p);
+  void write(const pen& p);
   
   void write(path p, bool newPath=true);
   
@@ -127,6 +235,11 @@ public:
     else *out << "stroke" << newl;
   }
   
+  void strokepath() {
+    if(pdf) reportError("PDF does not support strokepath");
+    else *out << "strokepath" << newl;
+  }
+  
   void fill(const pen &p) {
     if(p.evenodd()) {
       if(pdf) *out << "f*" << newl;
@@ -156,18 +269,19 @@ public:
   void latticeshade(const vm::array& a, const bbox& b);
   
   void gradientshade(bool axial, const ColorSpace& colorspace,
-		     const pen& pena, const pair& a, double ra,
-		     const pen& penb, const pair& b, double rb);
+                     const pen& pena, const pair& a, double ra,
+                     const pen& penb, const pair& b, double rb);
   
   void gouraudshade(const vm::array& pens, const vm::array& vertices,
-		    const vm::array& edges);
+                    const vm::array& edges);
   void tensorshade(const vm::array& pens, const vm::array& boundaries,
-		   const vm::array& z);
+                   const vm::array& z);
 
-  void imageheader(double width, double height, ColorSpace colorspace);
+  void imageheader(size_t width, size_t height, ColorSpace colorspace);
   
-  void image(const vm::array& a, const vm::array& p);
-  void image(const vm::array& a);
+  void image(const vm::array& a, const vm::array& p, bool antialias);
+  void image(const vm::array& a, bool antialias);
+  void rawimage(unsigned char *a, size_t width, size_t height, bool antialias);
 
   void gsave(bool tex=false) {
     if(pdf) *out << "q";
@@ -210,10 +324,10 @@ public:
   }
 
   // Determine shading and image transparency based on first pen.
-  void setfirstpen(const vm::array& pens) {
+  void setfirstopacity(const vm::array& pens) {
     if(pens.size() > 0) {
       pen *p=vm::read<pen *>(pens,0);
-      setpen(*p);
+      setopacity(*p);
     }
   }
   
@@ -233,7 +347,7 @@ public:
     size_t size=penarray.size();
     for(size_t i=0; i < size; i++)
       colorspace=max(colorspace,
-		     maxcolorspace(vm::read<vm::array>(penarray,i)));
+                     maxcolorspace(vm::read<vm::array>(penarray,i)));
     return colorspace;
   }
 
