@@ -15,7 +15,26 @@
 #include "item.h"
 
 namespace types {
-  class record;
+class record;
+}
+
+namespace camp {
+void glrenderWrapper();
+}
+
+namespace gl {
+extern bool glthread;
+extern bool initialize;
+
+#ifdef HAVE_LIBPTHREAD
+extern pthread_t mainthread;
+extern pthread_cond_t initSignal;
+extern pthread_mutex_t initLock;
+extern pthread_cond_t readySignal;
+extern pthread_mutex_t readyLock;
+void wait(pthread_cond_t& signal, pthread_mutex_t& lock);
+void endwait(pthread_cond_t& signal, pthread_mutex_t& lock);
+#endif
 }
 
 namespace settings {
@@ -23,14 +42,19 @@ extern const char PROGRAM[];
 extern const char VERSION[];
 extern const char BUGREPORT[];
 
-extern const string docdir;
+extern char *argv0;
+
+void Warn(const string& s);
+void noWarn(const string& s);
+string warn(const string& s);
+extern string systemDir;
+extern string docdir;
+extern const string dirsep;
   
 extern bool safe;
   
-extern bool globalwrite();
+bool globalwrite();
 
-enum origin {CENTER,BOTTOM,TOP,ZERO};
-  
 extern const string suffix;
 extern const string guisuffix;
   
@@ -68,20 +92,31 @@ Int getScroll();
   
 bool pdf(const string& texengine);
 bool latex(const string& texengine);
+bool context(const string& texengine);
   
 string nativeformat();
 string defaultformat();
   
+const char *beginput(const string& texengine);
+const char *endput(const string& texengine);
 const char *beginlabel(const string& texengine);
 const char *endlabel(const string& texengine);
 const char *rawpostscript(const string& texengine);
+const char *beginpicture(const string& texengine);
+const char *endpicture(const string& texengine);
 const char *beginspecial(const string& texengine);
 const char *endspecial();
   
 extern bool fataltex[];
 const char **texabort(const string& texengine);
   
-string texengine();
+string texcommand();
 string texprogram();
+  
+const double inches=72;
+const double cm=inches/2.54;
 }
+
+extern const char *SVN_REVISION;
+
 #endif
