@@ -27,6 +27,9 @@ string stripDir(string name);
 // Strip the file from a filename, returning the directory.
 string stripFile(string name);
 
+// Like stripTeXFile except return "" for context TeX engine.
+string stripTeXFile(string name);
+
 // Strip the extension from a filename.
 string stripExt(string name, const string& suffix="");
   
@@ -37,21 +40,24 @@ void checkLocal(string name);
   
 // Construct a filename from the original, adding aux at the end, and
 // changing the suffix.
-string buildname(string filename, string suffix="",
-		      string aux="", bool stripdir=true);
+string buildname(string filename, string suffix="", string aux="");
 
 // Construct an alternate filename for a temporary file in the current
 // directory.
 string auxname(string filename, string suffix="");
 
+// Return an argv array corresponding to the fields in command delimited
+// by spaces not within matching single quotes.
+char **args(const char *command, bool quiet=false);
+  
 // Similar to the standard system call except allows interrupts and does
 // not invoke a shell.
 int System(const char *command, int quiet=0, bool wait=true,
-	   const char *hint=NULL, const char *application="",
-	   int *pid=NULL);
+           const char *hint=NULL, const char *application="",
+           int *pid=NULL);
 int System(const ostringstream& command, int quiet=0, bool wait=true,
-	   const char *hint=NULL, const char *application="",
-	   int *pid=NULL); 
+           const char *hint=NULL, const char *application="",
+           int *pid=NULL); 
   
 #if defined(__DECCXX_LIBCXX_RH70)
 extern "C" int kill(pid_t pid, Int sig) throw();
@@ -77,10 +83,7 @@ extern "C" double tgamma(double x);
 extern "C" int snprintf(char *str, size_t size, const char *format,...);
 extern "C" int fileno(FILE *);
 extern "C" char *strptime(const char *s, const char *format, struct tm *tm);
-#endif
-
-#if defined(__CYGWIN__)
-#define ARG_MAX _POSIX_ARG_MAX
+extern "C" int setenv(const char *name, const char *value, int overwrite);
 #endif
 
 extern bool False;
@@ -98,6 +101,7 @@ extern char *startpath;
 void backslashToSlash(string& s);
 void spaceToUnderscore(string& s);
 string Getenv(const char *name, bool msdos);
+char *getPath(char *p=NULL);
 
 void execError(const char *command, const char *hint, const char *application);
   
@@ -118,6 +122,8 @@ inline Int Abs(Int x) {
 }
 
 unsigned unsignedcast(Int n);
+unsignedInt unsignedIntcast(Int n);
 int intcast(Int n);
-  
+Int Intcast(unsignedInt n);
+
 #endif
